from albibong.classes.event_handler.world_data_utils import WorldDataUtils
from albibong.classes.world_data import WorldData


def handle_event_character_equipment_changed(world_data: WorldData, parameters):
    if 2 in parameters:
        if parameters[0] in world_data.char_id_to_username:
            if world_data.char_id_to_username[parameters[0]] != "not initialized":
                char = world_data.characters[
                    world_data.char_id_to_username[parameters[0]]
                ]
                if char.username in world_data.party_members:
                    char.update_equipment(parameters[2])
                    WorldDataUtils.ws_update_damage_meter(world_data)
        else:
            world_data.change_equipment_log[parameters[0]] = parameters[2]



class CrcCalculator:
    @staticmethod
    def calculate(_bytes, length):
        result = 4294967295
        key = 3988292384

        for i in range(length):
            result ^= _bytes[i]

            for _ in range(8):
                if (result & 1) > 0:
                    result = result >> 1 ^ key
                else:
                    result >>= 1

        return result



class OperationResponse:
    def __init__(self, operation_code, return_code, debug_message, parameters):
        self.operation_code = operation_code
        self.return_code = return_code
        self.debug_message = debug_message
        self.parameters = parameters


"""Peewee migrations -- 001_init.py.

Some examples (model - class or model name)::

    > Model = migrator.orm['table_name']            # Return model in current state by name
    > Model = migrator.ModelClass                   # Return model in current state by name

    > migrator.sql(sql)                             # Run custom SQL
    > migrator.run(func, *args, **kwargs)           # Run python function with the given args
    > migrator.create_model(Model)                  # Create a model (could be used as decorator)
    > migrator.remove_model(model, cascade=True)    # Remove a model
    > migrator.add_fields(model, **fields)          # Add fields to a model
    > migrator.change_fields(model, **fields)       # Change fields
    > migrator.remove_fields(model, *field_names, cascade=True)
    > migrator.rename_field(model, old_field_name, new_field_name)
    > migrator.rename_table(model, new_table_name)
    > migrator.add_index(model, *col_names, unique=False)
    > migrator.add_not_null(model, *field_names)
    > migrator.add_default(model, field_name, default)
    > migrator.add_constraint(model, name, sql)
    > migrator.drop_index(model, *col_names)
    > migrator.drop_not_null(model, *field_names)
    > migrator.drop_constraints(model, *constraints)

"""

from contextlib import suppress

import peewee as pw
from peewee_migrate import Migrator

with suppress(ImportError):
    import playhouse.postgres_ext as pw_pext


def migrate(migrator: Migrator, database: pw.Database, *, fake=False):
    """Write your migrations here."""

    print("001_init")

    @migrator.create_model
    class BaseModel(pw.Model):
        id = pw.AutoField()

        class Meta:
            table_name = "basemodel"

    @migrator.create_model
    class Dungeon(pw.Model):
        id = pw.UUIDField(unique=True)
        type = pw.CharField(max_length=255)
        name = pw.CharField(max_length=255)
        tier = pw.IntegerField(default=0)
        fame = pw.FloatField(default=0.0)
        silver = pw.FloatField(default=0.0)
        re_spec = pw.FloatField(default=0.0)
        start_time = pw.DateTimeField()
        end_time = pw.DateTimeField(null=True)
        meter = pw.TextField()

        class Meta:
            table_name = "dungeon"

    @migrator.create_model
    class Island(pw.Model):
        uuid = pw.UUIDField(unique=True)
        id = pw.CharField(max_length=255)
        name = pw.CharField(max_length=255)
        type = pw.CharField(max_length=255)
        start_time = pw.DateTimeField()
        crops = pw.TextField()
        animals = pw.TextField()

        class Meta:
            table_name = "island"

    @migrator.create_model
    class Location(pw.Model):
        uuid = pw.UUIDField(unique=True)
        id = pw.CharField(max_length=255)
        name = pw.CharField(max_length=255)
        type = pw.CharField(max_length=255)

        class Meta:
            table_name = "location"


def rollback(migrator: Migrator, database: pw.Database, *, fake=False):
    """Write your rollback migrations here."""

    migrator.remove_model("location")

    migrator.remove_model("island")

    migrator.remove_model("dungeon")

    migrator.remove_model("basemodel")



from enum import Enum

class Protocol16Type(Enum):
    UNKNOWN = 0
    NULL = 42
    DICTIONARY = 68
    STRINGARRAY = 97
    BYTE = 98
    DOUBLE = 100
    EVENTDATA = 101
    FLOAT = 102
    INTEGER = 105
    HASHTABLE = 104
    SHORT = 107
    LONG = 108
    INTEGERARRAY = 110
    BOOLEAN = 111
    OPERATIONRESPONSE = 112
    OPERATIONREQUEST = 113 
    STRING = 115
    BYTEARRAY = 120
    ARRAY = 121
    OBJECTARRAY = 122


from albibong.classes.world_data import WorldData


def handle_operation_farmable_harvest(world_data: WorldData, parameters):
    if type(world_data.current_map).__name__ == "Island":
        for i in range(len(parameters[0])):
            world_data.current_map.add_crop(
                unique_name=parameters[0][i], quantity=parameters[1][i]
            )


def handle_operation_farmable_get_product(world_data: WorldData, parameters):
    if type(world_data.current_map).__name__ == "Island":
        for i in range(len(parameters[0])):
            world_data.current_map.add_animal(
                unique_name=parameters[0][i], quantity=parameters[1][i]
            )



import os

from peewee import *

home_dir = os.path.expanduser("~")
SQLITE_DB = f"{home_dir}/Albibong/Albibong.db"
db = SqliteDatabase(SQLITE_DB)


class BaseModel(Model):
    class Meta:
        database = db



import json
from datetime import datetime, timedelta
from uuid import UUID

from albibong.classes.coords import Coords
from albibong.classes.item import Item

DIVISOR = 10000


class Character:

    def __init__(
        self,
        id: int,
        uuid: UUID,
        username: str,
        guild: str,
        alliance: str,
        coords: Coords,
        equipment: list[Item] = [Item.get_item_from_code("0")] * 10,
    ):
        # Profile
        self.id = id
        self.uuid = uuid
        self.username = username
        self.guild = guild
        self.alliance = alliance
        self.coords = coords
        self.equipment = equipment

        # Stats
        self.fame_gained: int = 0
        self.re_spec_gained: int = 0
        self.silver_gained: int = 0
        self.loot: list[str] = []
        self.might_gained: int = 0
        self.favor_gained: int = 0

        # Combat
        self.damage_dealt: int = 0
        self.healing_dealt: int = 0
        self.is_already_in_combat: bool = False
        self.start_combat_time: timedelta = timedelta(0, 0)
        self.total_combat_duration: timedelta = timedelta(0, 0)

    def update_combat_duration(self, is_starting_combat):
        if self.is_already_in_combat == False:
            if is_starting_combat == True:
                self.is_already_in_combat = True
                self.start_combat_time = datetime.now()
        else:
            if is_starting_combat == False:
                self.is_already_in_combat = False
                current_combat_duration = datetime.now() - self.start_combat_time
                self.total_combat_duration += current_combat_duration

    def update_damage_dealt(self, nominal):
        if self.is_already_in_combat:
            self.damage_dealt += nominal

    def update_heal_dealt(self, nominal):
        if self.is_already_in_combat:
            self.healing_dealt += nominal

    def update_coords(self, parameters):
        if 3 in parameters:
            self.coords = Coords(parameters[3][0], parameters[3][1])

    def update_fame(self, parameters):
        fame = parameters[2] / DIVISOR
        self.fame_gained += fame

    def update_re_spec(self, parameters):
        re_spec = parameters[2] / DIVISOR
        self.re_spec_gained += re_spec

    def update_loot(self, parameters):
        if 3 in parameters and parameters[3] == True:
            if 5 in parameters:
                silver = parameters[5] / DIVISOR
                self.silver_gained += silver
        else:
            self.loot.append(parameters[4])

    def update_equipment(self, equipments):
        new_eq = []
        if equipments != []:
            for eq in equipments:
                obj = Item.get_item_from_code(str(eq))
                new_eq.append(obj)
        self.equipment = new_eq

    def update_might_and_favor(self, parameters):
        self.favor_gained += parameters[4] / DIVISOR
        self.might_gained += parameters[1] / DIVISOR



import io
from albibong.photon_packet_parser.message_type import MessageType
from albibong.photon_packet_parser.command_type import CommandType
from albibong.photon_packet_parser.segmented_packet import SegmentedPacket
from albibong.photon_packet_parser.protocol16_deserializer import Protocol16Deserializer
from albibong.photon_packet_parser.crc_calculator import CrcCalculator
from albibong.photon_packet_parser.number_serializer import NumberSerializer

COMMAND_HEADER_LENGTH = 12
PHOTON_HEADER_LENGTH = 12


class PhotonPacketParser:
    def __init__(self, on_event, on_request, on_response):
        self._pending_segments = {}
        self.on_event = on_event
        self.on_request = on_request
        self.on_response = on_response

    # MITIGATOR Challenge Response Packet
    def is_mcr_packet(self, payload):
        signatures = {
            b"\x4d\x43\x52\x48\x33\x31\x31\x30",
            b"\xe9\x71\x2d\xd5\x00\x01\x00\x00",
            b"\xe9\x71\x2d\xd5\x01\x01\x00\x00",
            b"\xe9\x71\x2d\xd5\x11\x01\x00\x00",
        }
        return payload[:8] in signatures

    def handle_payload(self, payload):
        if self.is_mcr_packet(payload):
            return

        payload = io.BytesIO(payload)
        if payload.getbuffer().nbytes < PHOTON_HEADER_LENGTH:
            return

        peer_id = NumberSerializer.deserialize_short(payload)
        flags = payload.read(1)[0]
        command_count = payload.read(1)[0]
        timestamp = NumberSerializer.deserialize_int(payload)
        challenge = NumberSerializer.deserialize_int(payload)

        is_encrypted = flags == 1
        is_crc_enabled = flags == 0xCC

        if is_encrypted:
            return

        if is_crc_enabled:
            offset = payload.tell()
            payload.seek(0)
            crc = NumberSerializer.deserialize_int(payload)

            payload.seek(offset)
            payload = NumberSerializer.serialize_int(0, payload)

            if crc != CrcCalculator.calculate(payload, payload.getbuffer().nbytes):
                return

        for _ in range(command_count):
            self.handle_command(payload)

    def handle_command(self, source: io.BytesIO):
        command_type = source.read(1)[0]
        channel_id = source.read(1)[0]
        command_flags = source.read(1)[0]
        # Skip 1 byte
        source.read(1)
        command_length = NumberSerializer.deserialize_int(source)
        sequence_number = NumberSerializer.deserialize_int(source)
        command_length -= COMMAND_HEADER_LENGTH

        if command_type == CommandType.Disconnect.value:
            return
        elif command_type == CommandType.SendUnreliable.value:
            source.read(4)
            command_length -= 4
            self.handle_send_reliable(source, command_length)
        elif command_type == CommandType.SendReliable.value:
            self.handle_send_reliable(source, command_length)
        elif command_type == CommandType.SendFragment.value:
            self.handle_send_fragment(source, command_length)
        else:
            source.read(command_length)

    def handle_send_reliable(self, source: io.BytesIO, command_length: int):
        # Skip 1 byte
        source.read(1)
        command_length -= 1
        message_type = source.read(1)[0]
        command_length -= 1

        operation_length = command_length
        payload = io.BytesIO(source.read(operation_length))

        if message_type == MessageType.OperationRequest.value:
            request_data = Protocol16Deserializer.deserialize_operation_request(payload)
            self.on_request(request_data)
        elif message_type == MessageType.OperationResponse.value:
            response_data = Protocol16Deserializer.deserialize_operation_response(
                payload
            )
            self.on_response(response_data)
        elif message_type == MessageType.Event.value:
            event_data = Protocol16Deserializer.deserialize_event_data(payload)
            self.on_event(event_data)
        # else:
        #     print("Unknown message type: ", message_type)

    def handle_send_fragment(self, source: io.BytesIO, command_length: int):
        start_sequence_number = NumberSerializer.deserialize_int(source)
        command_length -= 4
        fragment_count = NumberSerializer.deserialize_int(source)
        command_length -= 4
        fragment_number = NumberSerializer.deserialize_int(source)
        command_length -= 4
        total_length = NumberSerializer.deserialize_int(source)
        command_length -= 4
        fragment_offset = NumberSerializer.deserialize_int(source)
        command_length -= 4

        fragment_length = command_length

        self.handle_segmented_payload(
            start_sequence_number,
            total_length,
            fragment_length,
            fragment_offset,
            source,
        )

    def handle_finished_segmented_packet(self, total_payload: bytearray):
        command_length = len(total_payload)
        self.handle_send_reliable(io.BytesIO(total_payload), command_length)

    def handle_segmented_payload(
        self,
        start_sequence_number,
        total_length,
        fragment_length,
        fragment_offset,
        source,
    ):
        segmented_packet = self.get_segmented_packet(
            start_sequence_number, total_length
        )

        for i in range(fragment_length):
            segmented_packet.total_payload[fragment_offset + i] = source.read(1)[0]

        segmented_packet.bytes_written += fragment_length

        if segmented_packet.bytes_written >= segmented_packet.total_length:
            self._pending_segments.pop(start_sequence_number)
            self.handle_finished_segmented_packet(segmented_packet.total_payload)

    def get_segmented_packet(self, start_sequence_number, total_length):
        if start_sequence_number in self._pending_segments:
            return self._pending_segments[start_sequence_number]

        segmented_packet = SegmentedPacket(
            total_length=total_length, total_payload=bytearray(total_length)
        )

        self._pending_segments[start_sequence_number] = segmented_packet

        return segmented_packet



from albibong.classes.event_handler.world_data_utils import WorldDataUtils
from albibong.classes.location import Island, Location
from albibong.classes.world_data import WorldData
from albibong.threads.websocket_server import send_event


def handle_operation_change_cluster(world_data: WorldData, parameters):
    world_data.change_equipment_log = {}

    if type(world_data.current_map).__name__ == "Island":
        WorldDataUtils.set_island_status(
            current_island=world_data.current_map, parameters=parameters
        )

    if 1 in parameters:
        check_map = Location.get_location_from_code(parameters[1])
        map_type_splitted = set(check_map.type.split(" "))
        WorldDataUtils.set_dungeon_status(world_data, check_map, map_type_splitted)

        if "ISLAND" in map_type_splitted:
            island = Island.get_island_from_code(parameters[1])
            island.name = f"{parameters[2]}'s {island.name}"
            world_data.current_map = island
        elif "HIDEOUT" in map_type_splitted:
            hideout = Location.get_location_from_code(parameters[1])
            hideout.name = f"{parameters[2]}'s {hideout.name}"
            world_data.current_map = hideout

    elif 0 in parameters:
        check_map = Location.get_location_from_code(parameters[0])
        map_type_splitted = set(check_map.type.split(" "))
        is_dungeon = WorldDataUtils.set_dungeon_status(
            world_data, check_map, map_type_splitted
        )
        if is_dungeon == False:
            world_data.current_map = Location.get_location_from_code(parameters[0])

    WorldDataUtils.ws_update_location(world_data)



from albibong.classes.character import Character
from albibong.classes.coords import Coords
from albibong.classes.event_handler.world_data_utils import WorldDataUtils
from albibong.classes.utils import Utils
from albibong.classes.world_data import WorldData


def handle_event_new_character(world_data: WorldData, parameters):
    id = parameters[0]
    uuid = parameters[7]
    username = parameters[1]
    guild = parameters[8] if 8 in parameters else ""
    alliance = parameters[51] if 51 in parameters else ""
    coords = (
        Coords(parameters[15][0], parameters[15][1])
        if 15 in parameters
        else Coords(0, 0)
    )
    equipments = parameters[40] if 40 in parameters else []

    # initiate character
    if username not in world_data.characters:
        char: Character = Character(
            id=id,
            uuid=Utils.convert_int_arr_to_uuid(uuid),
            username=username,
            guild=guild,
            alliance=alliance,
            coords=coords,
        )
        char.update_equipment(equipments)
        world_data.characters[char.username] = char
        world_data.char_id_to_username[char.id] = char.username
        world_data.char_uuid_to_username[char.uuid] = char.username

    # change map
    else:
        char: Character = world_data.characters[username]
        char.update_equipment(equipments)
        char.coords = coords
        WorldDataUtils.convert_id_to_name(
            world_data, old_id=char.id, new_id=id, char=char
        )



from albibong.photon_packet_parser.photon_packet_parser import PhotonPacketParser
from albibong.photon_packet_parser.event_data import EventData
from albibong.photon_packet_parser.operation_request import OperationRequest
from albibong.photon_packet_parser.operation_response import OperationResponse



from albibong.classes.event_handler.handle_event_character_equipment_changed import (
    handle_event_character_equipment_changed,
)
from albibong.classes.event_handler.handle_event_health import (
    handle_event_health_update,
    handle_event_health_updates,
)
from albibong.classes.event_handler.handle_event_in_combat_state_update import (
    handle_event_in_combat_state_update,
)
from albibong.classes.event_handler.handle_event_might_and_favor_received_event import (
    handle_event_might_and_favor_received_event,
)
from albibong.classes.event_handler.handle_event_new_character import (
    handle_event_new_character,
)
from albibong.classes.event_handler.handle_event_other_grabbed_loot import (
    handle_event_other_grabbed_loot,
)
from albibong.classes.event_handler.handle_event_party import (
    handle_event_party_disbanded,
    handle_event_party_joined,
    handle_event_party_player_joined,
    handle_event_party_player_left,
)
from albibong.classes.event_handler.handle_event_update_fame import (
    handle_event_update_fame,
)
from albibong.classes.event_handler.handle_event_update_re_spec_points import (
    handle_event_update_re_spec_points,
)
from albibong.classes.event_handler.handle_operation_change_cluster import (
    handle_operation_change_cluster,
)
from albibong.classes.event_handler.handle_operation_farmable_harvest import (
    handle_operation_farmable_get_product,
    handle_operation_farmable_harvest,
)
from albibong.classes.event_handler.handle_operation_join import handle_operation_join
from albibong.classes.event_handler.handle_operation_move import handle_operation_move
from albibong.classes.world_data import WorldData
from albibong.resources.EventCode import EventCode
from albibong.resources.OperationCode import OperationCode

EVENT_TYPE_PARAMETER = 252
REQUEST_TYPE_PARAMETER = 253
RESPONSE_TYPE_PARAMETER = 253


class EventHandler:
    def __init__(self):
        self.request_handler = {}
        self.response_handler = {}
        self.event_handler = {}

        # Event Handler
        self.event_handler[EventCode.NEW_CHARACTER.value] = handle_event_new_character
        self.event_handler[EventCode.HEALTH_UPDATE.value] = handle_event_health_update
        self.event_handler[EventCode.HEALTH_UPDATES.value] = handle_event_health_updates
        self.event_handler[EventCode.IN_COMBAT_STATE_UPDATE.value] = (
            handle_event_in_combat_state_update
        )

        self.event_handler[EventCode.UPDATE_FAME.value] = handle_event_update_fame
        self.event_handler[EventCode.UPDATE_RE_SPEC_POINTS.value] = (
            handle_event_update_re_spec_points
        )
        self.event_handler[EventCode.MIGHT_AND_FAVOR_RECEIVED_EVENT.value] = (
            handle_event_might_and_favor_received_event
        )

        self.event_handler[EventCode.OTHER_GRABBED_LOOT.value] = (
            handle_event_other_grabbed_loot
        )
        self.event_handler[EventCode.PARTY_JOINED.value] = handle_event_party_joined
        self.event_handler[EventCode.PARTY_DISBANDED.value] = (
            handle_event_party_disbanded
        )
        self.event_handler[EventCode.PARTY_PLAYER_JOINED.value] = (
            handle_event_party_player_joined
        )
        self.event_handler[EventCode.PARTY_PLAYER_LEFT.value] = (
            handle_event_party_player_left
        )
        self.event_handler[EventCode.CHARACTER_EQUIPMENT_CHANGED.value] = (
            handle_event_character_equipment_changed
        )

        # Request Handler
        self.request_handler[OperationCode.MOVE.value] = handle_operation_move

        # Response Handler
        self.response_handler[OperationCode.JOIN.value] = handle_operation_join
        self.response_handler[OperationCode.CHANGE_CLUSTER.value] = (
            handle_operation_change_cluster
        )
        self.response_handler[OperationCode.FARMABLE_GET_PRODUCT.value] = (
            handle_operation_farmable_get_product
        )
        self.response_handler[OperationCode.FARMABLE_HARVEST.value] = (
            handle_operation_farmable_harvest
        )

    def on_request(self, world_data: WorldData, parameters):
        if REQUEST_TYPE_PARAMETER not in parameters:
            return None

        if parameters[REQUEST_TYPE_PARAMETER] not in self.request_handler:
            return None

        handler = self.request_handler[parameters[REQUEST_TYPE_PARAMETER]]
        return handler(world_data, parameters)

    def on_response(self, world_data: WorldData, parameters):
        if RESPONSE_TYPE_PARAMETER not in parameters:
            return None

        if parameters[RESPONSE_TYPE_PARAMETER] not in self.response_handler:
            return None

        handler = self.response_handler[parameters[RESPONSE_TYPE_PARAMETER]]
        return handler(world_data, parameters)

    def on_event(self, world_data: WorldData, parameters):
        if EVENT_TYPE_PARAMETER not in parameters:
            return None

        if parameters[EVENT_TYPE_PARAMETER] not in self.event_handler:
            return None

        handler = self.event_handler[parameters[EVENT_TYPE_PARAMETER]]
        return handler(world_data, parameters)



from enum import Enum


class EventCode(Enum):

    LEAVE = 1
    JOIN_FINISHED = 2
    MOVE = 3
    TELEPORT = 4
    CHANGE_EQUIPMENT = 5
    HEALTH_UPDATE = 6
    HEALTH_UPDATES = 7
    ENERGY_UPDATE = 8
    DAMAGE_SHIELD_UPDATE = 9
    CRAFTING_FOCUS_UPDATE = 10
    ACTIVE_SPELL_EFFECTS_UPDATE = 11
    RESET_COOLDOWNS = 12
    ATTACK = 13
    CAST_START = 14
    CHANNELING_UPDATE = 15
    CAST_CANCEL = 16
    CAST_TIME_UPDATE = 17
    CAST_FINISHED = 18
    CAST_SPELL = 19
    CAST_SPELLS = 20
    CAST_HIT = 21
    CAST_HITS = 22
    STORED_TARGETS_UPDATE = 23
    CHANNELING_ENDED = 24
    ATTACK_BUILDING = 25
    INVENTORY_PUT_ITEM = 26
    INVENTORY_DELETE_ITEM = 27
    INVENTORY_STATE = 28
    NEW_CHARACTER = 29
    NEW_EQUIPMENT_ITEM = 30
    NEW_SIEGE_BANNER_ITEM = 31
    NEW_SIMPLE_ITEM = 32
    NEW_FURNITURE_ITEM = 33
    NEW_KILL_TROPHY_ITEM = 34
    NEW_JOURNAL_ITEM = 35
    NEW_LABORER_ITEM = 36
    NEW_EQUIPMENT_ITEM_LEGENDARY_SOUL = 37
    NEW_SIMPLE_HARVESTABLE_OBJECT = 38
    NEW_SIMPLE_HARVESTABLE_OBJECT_LIST = 39
    NEW_HARVESTABLE_OBJECT = 40
    NEW_TREASURE_DESTINATION_OBJECT = 41
    TREASURE_DESTINATION_OBJECT_STATUS = 42
    CLOSE_TREASURE_DESTINATION_OBJECT = 43
    NEW_SILVER_OBJECT = 44
    NEW_BUILDING = 45
    HARVESTABLE_CHANGE_STATE = 46
    MOB_CHANGE_STATE = 47
    FACTION_BUILDING_INFO = 48
    CRAFT_BUILDING_INFO = 49
    REPAIR_BUILDING_INFO = 50
    MELD_BUILDING_INFO = 51
    CONSTRUCTION_SITE_INFO = 52
    PLAYER_BUILDING_INFO = 53
    FARM_BUILDING_INFO = 54
    TUTORIAL_BUILDING_INFO = 55
    LABORER_OBJECT_INFO = 56
    LABORER_OBJECT_JOB_INFO = 57
    MARKET_PLACE_BUILDING_INFO = 58
    HARVEST_START = 59
    HARVEST_CANCEL = 60
    HARVEST_FINISHED = 61
    TAKE_SILVER = 62
    REMOVE_SILVER = 63
    ACTION_ON_BUILDING_START = 64
    ACTION_ON_BUILDING_CANCEL = 65
    ACTION_ON_BUILDING_FINISHED = 66
    ITEM_REROLL_QUALITY_FINISHED = 67
    INSTALL_RESOURCE_START = 68
    INSTALL_RESOURCE_CANCEL = 69
    INSTALL_RESOURCE_FINISHED = 70
    CRAFT_ITEM_FINISHED = 71
    LOGOUT_CANCEL = 72
    CHAT_MESSAGE = 73
    CHAT_SAY = 74
    CHAT_WHISPER = 75
    CHAT_MUTED = 76
    PLAY_EMOTE = 77
    STOP_EMOTE = 78
    SYSTEM_MESSAGE = 79
    UTILITY_TEXT_MESSAGE = 80
    UPDATE_MONEY = 81
    UPDATE_FAME = 82
    UPDATE_LEARNING_POINTS = 83
    UPDATE_RE_SPEC_POINTS = 84
    UPDATE_CURRENCY = 85
    UPDATE_FACTION_STANDING = 86
    UPDATE_MIST_CITY_STANDING = 87
    RESPAWN = 88
    SERVER_DEBUG_LOG = 89
    CHARACTER_EQUIPMENT_CHANGED = 90
    REGENERATION_HEALTH_CHANGED = 91
    REGENERATION_ENERGY_CHANGED = 92
    REGENERATION_MOUNT_HEALTH_CHANGED = 93
    REGENERATION_CRAFTING_CHANGED = 94
    REGENERATION_HEALTH_ENERGY_COMBO_CHANGED = 95
    REGENERATION_PLAYER_COMBO_CHANGED = 96
    DURABILITY_CHANGED = 97
    NEW_LOOT = 98
    ATTACH_ITEM_CONTAINER = 99
    DETACH_ITEM_CONTAINER = 100
    INVALIDATE_ITEM_CONTAINER = 101
    LOCK_ITEM_CONTAINER = 102
    GUILD_UPDATE = 103
    GUILD_PLAYER_UPDATED = 104
    INVITED_TO_GUILD = 105
    GUILD_MEMBER_WORLD_UPDATE = 106
    UPDATE_MATCH_DETAILS = 107
    OBJECT_EVENT = 108
    NEW_MONOLITH_OBJECT = 109
    MONOLITH_HAS_BANNERS_PLACED_UPDATE = 110
    NEW_ORB_OBJECT = 111
    NEW_CASTLE_OBJECT = 112
    NEW_SPELL_EFFECT_AREA = 113
    UPDATE_SPELL_EFFECT_AREA = 114
    NEW_CHAIN_SPELL = 115
    UPDATE_CHAIN_SPELL = 116
    NEW_TREASURE_CHEST = 117
    START_MATCH = 118
    START_ARENA_MATCH_INFOS = 119
    END_ARENA_MATCH = 120
    MATCH_UPDATE = 121
    ACTIVE_MATCH_UPDATE = 122
    NEW_MOB = 123
    DEBUG_MOB_INFO = 124
    DEBUG_VARIABLES_INFO = 125
    DEBUG_REPUTATION_INFO = 126
    DEBUG_DIMINISHING_RETURN_INFO = 127
    DEBUG_SMART_CLUSTER_QUEUE_INFO = 128
    CLAIM_ORB_START = 129
    CLAIM_ORB_FINISHED = 130
    CLAIM_ORB_CANCEL = 131
    ORB_UPDATE = 132
    ORB_CLAIMED = 133
    ORB_RESET = 134
    NEW_WAR_CAMP_OBJECT = 135
    NEW_MATCH_LOOT_CHEST_OBJECT = 136
    NEW_ARENA_EXIT = 137
    GUILD_MEMBER_TERRITORY_UPDATE = 138
    INVITED_MERCENARY_TO_MATCH = 139
    CLUSTER_INFO_UPDATE = 140
    FORCED_MOVEMENT = 141
    FORCED_MOVEMENT_CANCEL = 142
    CHARACTER_STATS = 143
    CHARACTER_STATS_KILL_HISTORY = 144
    CHARACTER_STATS_DEATH_HISTORY = 145
    GUILD_STATS = 146
    KILL_HISTORY_DETAILS = 147
    FULL_ACHIEVEMENT_INFO = 148
    FINISHED_ACHIEVEMENT = 149
    ACHIEVEMENT_PROGRESS_INFO = 150
    FULL_ACHIEVEMENT_PROGRESS_INFO = 151
    FULL_TRACKED_ACHIEVEMENT_INFO = 152
    FULL_AUTO_LEARN_ACHIEVEMENT_INFO = 153
    QUEST_GIVER_QUEST_OFFERED = 154
    QUEST_GIVER_DEBUG_INFO = 155
    CONSOLE_EVENT = 156
    TIME_SYNC = 157
    CHANGE_AVATAR = 158
    CHANGE_MOUNT_SKIN = 159
    GAME_EVENT = 160
    KILLED_PLAYER = 161
    DIED = 162
    KNOCKED_DOWN = 163
    UNCONCIOUS = 164
    MATCH_PLAYER_JOINED_EVENT = 165
    MATCH_PLAYER_STATS_EVENT = 166
    MATCH_PLAYER_STATS_COMPLETE_EVENT = 167
    MATCH_TIME_LINE_EVENT_EVENT = 168
    MATCH_PLAYER_MAIN_GEAR_STATS_EVENT = 169
    MATCH_PLAYER_CHANGED_AVATAR_EVENT = 170
    INVITATION_PLAYER_TRADE = 171
    PLAYER_TRADE_START = 172
    PLAYER_TRADE_CANCEL = 173
    PLAYER_TRADE_UPDATE = 174
    PLAYER_TRADE_FINISHED = 175
    PLAYER_TRADE_ACCEPT_CHANGE = 176
    MINI_MAP_PING = 177
    MARKET_PLACE_NOTIFICATION = 178
    DUELLING_CHALLENGE_PLAYER = 179
    NEW_DUELLING_POST = 180
    DUEL_STARTED = 181
    DUEL_ENDED = 182
    DUEL_DENIED = 183
    DUEL_REQUEST_CANCELED = 184
    DUEL_LEFT_AREA = 185
    DUEL_RE_ENTERED_AREA = 186
    NEW_REAL_ESTATE = 187
    MINI_MAP_OWNED_BUILDINGS_POSITIONS = 188
    REAL_ESTATE_LIST_UPDATE = 189
    GUILD_LOGO_UPDATE = 190
    GUILD_LOGO_CHANGED = 191
    PLACEABLE_OBJECT_PLACE = 192
    PLACEABLE_OBJECT_PLACE_CANCEL = 193
    FURNITURE_OBJECT_BUFF_PROVIDER_INFO = 194
    FURNITURE_OBJECT_CHEAT_PROVIDER_INFO = 195
    FARMABLE_OBJECT_INFO = 196
    NEW_UNREAD_MAILS = 197
    MAIL_OPERATION_POSSIBLE = 198
    GUILD_LOGO_OBJECT_UPDATE = 199
    START_LOGOUT = 200
    NEW_CHAT_CHANNELS = 201
    JOINED_CHAT_CHANNEL = 202
    LEFT_CHAT_CHANNEL = 203
    REMOVED_CHAT_CHANNEL = 204
    ACCESS_STATUS = 205
    MOUNTED = 206
    MOUNT_START = 207
    MOUNT_CANCEL = 208
    NEW_TRAVELPOINT = 209
    NEW_ISLAND_ACCESS_POINT = 210
    NEW_EXIT = 211
    UPDATE_HOME = 212
    UPDATE_CHAT_SETTINGS = 213
    RESURRECTION_OFFER = 214
    RESURRECTION_REPLY = 215
    LOOT_EQUIPMENT_CHANGED = 216
    UPDATE_UNLOCKED_GUILD_LOGOS = 217
    UPDATE_UNLOCKED_AVATARS = 218
    UPDATE_UNLOCKED_AVATAR_RINGS = 219
    UPDATE_UNLOCKED_BUILDINGS = 220
    NEW_ISLAND_MANAGEMENT = 221
    NEW_TELEPORT_STONE = 222
    CLOAK = 223
    PARTY_INVITATION = 224
    PARTY_JOIN_REQUEST = 225
    PARTY_JOINED = 226
    PARTY_DISBANDED = 227
    PARTY_PLAYER_JOINED = 228
    PARTY_CHANGED_ORDER = 229
    PARTY_PLAYER_LEFT = 230
    PARTY_LEADER_CHANGED = 231
    PARTY_LOOT_SETTING_CHANGED_PLAYER = 232
    PARTY_SILVER_GAINED = 233
    PARTY_PLAYER_UPDATED = 234
    PARTY_INVITATION_ANSWER = 235
    PARTY_JOIN_REQUEST_ANSWER = 236
    PARTY_MARKED_OBJECTS_UPDATED = 237
    PARTY_ON_CLUSTER_PARTY_JOINED = 238
    PARTY_SET_ROLE_FLAG = 239
    PARTY_INVITE_OR_JOIN_PLAYER_EQUIPMENT_INFO = 240
    PARTY_READY_CHECK_UPDATE = 241
    SPELL_COOLDOWN_UPDATE = 242
    NEW_HELLGATE_EXIT_PORTAL = 243
    NEW_EXPEDITION_EXIT = 244
    NEW_EXPEDITION_NARRATOR = 245
    EXIT_ENTER_START = 246
    EXIT_ENTER_CANCEL = 247
    EXIT_ENTER_FINISHED = 248
    NEW_QUEST_GIVER_OBJECT = 249
    FULL_QUEST_INFO = 250
    QUEST_PROGRESS_INFO = 251
    QUEST_GIVER_INFO_FOR_PLAYER = 252
    FULL_EXPEDITION_INFO = 253
    EXPEDITION_QUEST_PROGRESS_INFO = 254
    INVITED_TO_EXPEDITION = 255
    EXPEDITION_REGISTRATION_INFO = 256
    ENTERING_EXPEDITION_START = 257
    ENTERING_EXPEDITION_CANCEL = 258
    REWARD_GRANTED = 259
    ARENA_REGISTRATION_INFO = 260
    ENTERING_ARENA_START = 261
    ENTERING_ARENA_CANCEL = 262
    ENTERING_ARENA_LOCK_START = 263
    ENTERING_ARENA_LOCK_CANCEL = 264
    INVITED_TO_ARENA_MATCH = 265
    USING_HELLGATE_SHRINE = 266
    ENTERING_HELLGATE_LOCK_START = 267
    ENTERING_HELLGATE_LOCK_CANCEL = 268
    PLAYER_COUNTS = 269
    IN_COMBAT_STATE_UPDATE = 270
    OTHER_GRABBED_LOOT = 271
    TREASURE_CHEST_USING_START = 272
    TREASURE_CHEST_USING_FINISHED = 273
    TREASURE_CHEST_USING_CANCEL = 274
    TREASURE_CHEST_USING_OPENING_COMPLETE = 275
    TREASURE_CHEST_FORCE_CLOSE_INVENTORY = 276
    LOCAL_TREASURES_UPDATE = 277
    LOOT_CHEST_SPAWNPOINTS_UPDATE = 278
    PREMIUM_CHANGED = 279
    PREMIUM_EXTENDED = 280
    PREMIUM_LIFE_TIME_REWARD_GAINED = 281
    GOLD_PURCHASED = 282
    LABORER_GOT_UPGRADED = 283
    JOURNAL_GOT_FULL = 284
    JOURNAL_FILL_ERROR = 285
    FRIEND_REQUEST = 286
    FRIEND_REQUEST_INFOS = 287
    FRIEND_INFOS = 288
    FRIEND_REQUEST_ANSWERED = 289
    FRIEND_ONLINE_STATUS = 290
    FRIEND_REQUEST_CANCELED = 291
    FRIEND_REMOVED = 292
    FRIEND_UPDATED = 293
    PARTY_LOOT_ITEMS = 294
    PARTY_LOOT_ITEMS_REMOVED = 295
    REPUTATION_UPDATE = 296
    DEFENSE_UNIT_ATTACK_BEGIN = 297
    DEFENSE_UNIT_ATTACK_END = 298
    DEFENSE_UNIT_ATTACK_DAMAGE = 299
    UNRESTRICTED_PVP_ZONE_UPDATE = 300
    REPUTATION_IMPLICATION_UPDATE = 301
    NEW_MOUNT_OBJECT = 302
    MOUNT_HEALTH_UPDATE = 303
    MOUNT_COOLDOWN_UPDATE = 304
    NEW_EXPEDITION_AGENT = 305
    NEW_EXPEDITION_CHECK_POINT = 306
    EXPEDITION_START_EVENT = 307
    VOTE_EVENT = 308
    RATING_EVENT = 309
    NEW_ARENA_AGENT = 310
    BOOST_FARMABLE = 311
    USE_FUNCTION = 312
    NEW_PORTAL_ENTRANCE = 313
    NEW_PORTAL_EXIT = 314
    NEW_RANDOM_DUNGEON_EXIT = 315
    WAITING_QUEUE_UPDATE = 316
    PLAYER_MOVEMENT_RATE_UPDATE = 317
    OBSERVE_START = 318
    MINIMAP_ZERGS = 319
    MINIMAP_SMART_CLUSTER_ZERGS = 320
    PAYMENT_TRANSACTIONS = 321
    PERFORMANCE_STATS_UPDATE = 322
    OVERLOAD_MODE_UPDATE = 323
    DEBUG_DRAW_EVENT = 324
    RECORD_CAMERA_MOVE = 325
    RECORD_START = 326
    DELIVER_CARRIABLE_OBJECT_START = 327
    DELIVER_CARRIABLE_OBJECT_CANCEL = 328
    DELIVER_CARRIABLE_OBJECT_RESET = 329
    DELIVER_CARRIABLE_OBJECT_FINISHED = 330
    TERRITORY_CLAIM_START = 331
    TERRITORY_CLAIM_CANCEL = 332
    TERRITORY_CLAIM_FINISHED = 333
    TERRITORY_SCHEDULE_RESULT = 334
    TERRITORY_UPGRADE_WITH_POWER_CRYSTAL_RESULT = 335
    RECEIVE_CARRIABLE_OBJECT_START = 336
    RECEIVE_CARRIABLE_OBJECT_FINISHED = 337
    UPDATE_ACCOUNT_STATE = 338
    START_DETERMINISTIC_ROAM = 339
    GUILD_FULL_ACCESS_TAGS_UPDATED = 340
    GUILD_ACCESS_TAG_UPDATED = 341
    GVG_SEASON_UPDATE = 342
    GVG_SEASON_CHEAT_COMMAND = 343
    SEASON_POINTS_BY_KILLING_BOOSTER = 344
    FISHING_START = 345
    FISHING_CAST = 346
    FISHING_CATCH = 347
    FISHING_FINISHED = 348
    FISHING_CANCEL = 349
    NEW_FLOAT_OBJECT = 350
    NEW_FISHING_ZONE_OBJECT = 351
    FISHING_MINI_GAME = 352
    ALBION_JOURNAL_ACHIEVEMENT_COMPLETED = 353
    UPDATE_PUPPET = 354
    CHANGE_FLAGGING_FINISHED = 355
    NEW_OUTPOST_OBJECT = 356
    OUTPOST_UPDATE = 357
    OUTPOST_CLAIMED = 358
    OVER_CHARGE_END = 359
    OVER_CHARGE_STATUS = 360
    PARTY_FINDER_FULL_UPDATE = 361
    PARTY_FINDER_UPDATE = 362
    PARTY_FINDER_APPLICANTS_UPDATE = 363
    PARTY_FINDER_EQUIPMENT_SNAPSHOT = 364
    PARTY_FINDER_JOIN_REQUEST_DECLINED = 365
    NEW_UNLOCKED_PERSONAL_SEASON_REWARDS = 366
    PERSONAL_SEASON_POINTS_GAINED = 367
    PERSONAL_SEASON_PAST_SEASON_DATA_EVENT = 368
    EASY_ANTI_CHEAT_MESSAGE_TO_CLIENT = 369
    MATCH_LOOT_CHEST_OPENING_START = 370
    MATCH_LOOT_CHEST_OPENING_FINISHED = 371
    MATCH_LOOT_CHEST_OPENING_CANCEL = 372
    NOTIFY_CRYSTAL_MATCH_REWARD = 373
    CRYSTAL_REALM_FEEDBACK = 374
    NEW_LOCATION_MARKER = 375
    NEW_TUTORIAL_BLOCKER = 376
    NEW_TILE_SWITCH = 377
    NEW_INFORMATION_PROVIDER = 378
    NEW_DYNAMIC_GUILD_LOGO = 379
    NEW_DECORATION = 380
    TUTORIAL_UPDATE = 381
    TRIGGER_HINT_BOX = 382
    RANDOM_DUNGEON_POSITION_INFO = 383
    NEW_LOOT_CHEST = 384
    UPDATE_LOOT_CHEST = 385
    LOOT_CHEST_OPENED = 386
    UPDATE_LOOT_PROTECTED_BY_MOBS_WITH_MINIMAP_DISPLAY = 387
    NEW_SHRINE = 388
    UPDATE_SHRINE = 389
    UPDATE_ROOM = 390
    NEW_MIST_DUNGEON_ROOM_MOB_SOUL = 391
    NEW_HELLGATE_SHRINE = 392
    UPDATE_HELLGATE_SHRINE = 393
    ACTIVATE_HELLGATE_EXIT = 394
    MUTE_PLAYER_UPDATE = 395
    SHOP_TILE_UPDATE = 396
    SHOP_UPDATE = 397
    ANTI_CHEAT_KICK = 398
    BATTL_EYE_SERVER_MESSAGE = 399
    UNLOCK_VANITY_UNLOCK = 400
    AVATAR_UNLOCKED = 401
    CUSTOMIZATION_CHANGED = 402
    BASE_VAULT_INFO = 403
    GUILD_VAULT_INFO = 404
    BANK_VAULT_INFO = 405
    RECOVERY_VAULT_PLAYER_INFO = 406
    RECOVERY_VAULT_GUILD_INFO = 407
    UPDATE_WARDROBE = 408
    CASTLE_PHASE_CHANGED = 409
    GUILD_ACCOUNT_LOG_EVENT = 410
    NEW_HIDEOUT_OBJECT = 411
    NEW_HIDEOUT_MANAGEMENT = 412
    NEW_HIDEOUT_EXIT = 413
    INIT_HIDEOUT_ATTACK_START = 414
    INIT_HIDEOUT_ATTACK_CANCEL = 415
    INIT_HIDEOUT_ATTACK_FINISHED = 416
    HIDEOUT_MANAGEMENT_UPDATE = 417
    HIDEOUT_UPGRADE_WITH_POWER_CRYSTAL_RESULT = 418
    IP_CHANGED = 419
    SMART_CLUSTER_QUEUE_UPDATE_INFO = 420
    SMART_CLUSTER_QUEUE_ACTIVE_INFO = 421
    SMART_CLUSTER_QUEUE_KICK_WARNING = 422
    SMART_CLUSTER_QUEUE_INVITE = 423
    RECEIVED_GVG_SEASON_POINTS = 424
    TOWER_POWER_POINT_UPDATE = 425
    OPEN_WORLD_ATTACK_SCHEDULE_START = 426
    OPEN_WORLD_ATTACK_SCHEDULE_FINISHED = 427
    OPEN_WORLD_ATTACK_SCHEDULE_CANCEL = 428
    OPEN_WORLD_ATTACK_CONQUER_START = 429
    OPEN_WORLD_ATTACK_CONQUER_FINISHED = 430
    OPEN_WORLD_ATTACK_CONQUER_CANCEL = 431
    OPEN_WORLD_ATTACK_CONQUER_STATUS = 432
    OPEN_WORLD_ATTACK_START = 433
    OPEN_WORLD_ATTACK_END = 434
    NEW_RANDOM_RESOURCE_BLOCKER = 435
    NEW_HOME_OBJECT = 436
    HIDEOUT_OBJECT_UPDATE = 437
    UPDATE_INFAMY = 438
    MINIMAP_POSITION_MARKERS = 439
    NEW_TUNNEL_EXIT = 440
    CORRUPTED_DUNGEON_UPDATE = 441
    CORRUPTED_DUNGEON_STATUS = 442
    CORRUPTED_DUNGEON_INFAMY = 443
    HELLGATE_RESTRICTED_AREA_UPDATE = 444
    HELLGATE_INFAMY = 445
    HELLGATE_STATUS = 446
    HELLGATE_STATUS_UPDATE = 447
    HELLGATE_SUSPENSE = 448
    REPLACE_SPELL_SLOT_WITH_MULTI_SPELL = 449
    NEW_CORRUPTED_SHRINE = 450
    UPDATE_CORRUPTED_SHRINE = 451
    CORRUPTED_SHRINE_USAGE_START = 452
    CORRUPTED_SHRINE_USAGE_CANCEL = 453
    EXIT_USED = 454
    LINKED_TO_OBJECT = 455
    LINK_TO_OBJECT_BROKEN = 456
    ESTIMATED_MARKET_VALUE_UPDATE = 457
    STUCK_CANCEL = 458
    DUNGON_ESCAPE_READY = 459
    FACTION_WARFARE_CLUSTER_STATE = 460
    FACTION_WARFARE_HAS_UNCLAIMED_WEEKLY_REPORTS_EVENT = 461
    SIMPLE_FEEDBACK = 462
    SMART_CLUSTER_QUEUE_SKIP_CLUSTER_ERROR = 463
    XIGN_CODE_EVENT = 464
    BATCH_USE_ITEM_START = 465
    BATCH_USE_ITEM_END = 466
    RED_ZONE_EVENT_CLUSTER_STATUS = 467
    RED_ZONE_PLAYER_NOTIFICATION = 468
    RED_ZONE_WORLD_EVENT = 469
    FACTION_WARFARE_STATS = 470
    UPDATE_FACTION_BALANCE_FACTORS = 471
    FACTION_ENLISTMENT_CHANGED = 472
    UPDATE_FACTION_RANK = 473
    FACTION_WARFARE_CAMPAIGN_REWARDS_UNLOCKED = 474
    FEATURED_FEATURE_UPDATE = 475
    NEW_CARRIABLE_OBJECT = 476
    MINIMAP_CRYSTAL_POSITION_MARKER = 477
    CARRIED_OBJECT_UPDATE = 478
    PICKUP_CARRIABLE_OBJECT_START = 479
    PICKUP_CARRIABLE_OBJECT_CANCEL = 480
    PICKUP_CARRIABLE_OBJECT_FINISHED = 481
    DO_SIMPLE_ACTION_START = 482
    DO_SIMPLE_ACTION_CANCEL = 483
    DO_SIMPLE_ACTION_FINISHED = 484
    NOTIFY_GUEST_ACCOUNT_VERIFIED = 485
    MIGHT_AND_FAVOR_RECEIVED_EVENT = 486
    WEEKLY_PVP_CHALLENGE_REWARD_STATE_UPDATE = 487
    NEW_UNLOCKED_PVP_SEASON_CHALLENGE_REWARDS = 488
    STATIC_DUNGEON_ENTRANCES_DUNGEON_EVENT_STATUS_UPDATES = 489
    STATIC_DUNGEON_DUNGEON_VALUE_UPDATE = 490
    STATIC_DUNGEON_ENTRANCE_DUNGEON_EVENTS_ABORTED = 491
    IN_APP_PURCHASE_CONFIRMED_GOOGLE_PLAY = 492
    FEATURE_SWITCH_INFO = 493
    PARTY_JOIN_REQUEST_ABORTED = 494
    PARTY_INVITE_ABORTED = 495
    PARTY_START_HUNT_REQUEST = 496
    PARTY_START_HUNT_REQUESTED = 497
    PARTY_START_HUNT_REQUEST_ANSWER = 498
    PARTY_PLAYER_LEAVE_SCHEDULED = 499
    GUILD_INVITE_DECLINED = 500
    CANCEL_MULTI_SPELL_SLOTS = 501
    NEW_VISUAL_EVENT_OBJECT = 502
    CASTLE_CLAIM_PROGRESS = 503
    CASTLE_CLAIM_PROGRESS_LOGO = 504
    TOWN_PORTAL_UPDATE_STATE = 505
    TOWN_PORTAL_FAILED = 506
    CONSUMABLE_VANITY_CHARGES_ADDED = 507
    FESTIVITIES_UPDATE = 508
    NEW_BANNER_OBJECT = 509
    NEW_MISTS_IMMEDIATE_RETURN_EXIT = 510
    MISTS_PLAYER_JOINED_INFO = 511
    NEW_MISTS_STATIC_ENTRANCE = 512
    NEW_MISTS_OPEN_WORLD_EXIT = 513
    NEW_TUNNEL_EXIT_TEMP = 514
    NEW_MISTS_WISP_SPAWN = 515
    MISTS_WISP_SPAWN_STATE_CHANGE = 516
    NEW_MISTS_CITY_ENTRANCE = 517
    NEW_MISTS_CITY_ROADS_ENTRANCE = 518
    MISTS_CITY_ROADS_ENTRANCE_PARTY_STATE_UPDATE = 519
    MISTS_CITY_ROADS_ENTRANCE_CLEAR_STATE_FOR_PARTY = 520
    MISTS_ENTRANCE_DATA_CHANGED = 521
    NEW_MISTS_CAGED_WISP = 522
    MISTS_WISP_CAGE_OPENED = 523
    MISTS_ENTRANCE_PARTY_BINDING_CREATED = 524
    MISTS_ENTRANCE_PARTY_BINDING_CLEARED = 525
    MISTS_ENTRANCE_PARTY_BINDING_INFOS = 526
    NEW_MISTS_BORDER_EXIT = 527
    NEW_MISTS_DUNGEON_EXIT = 528
    LOCAL_QUEST_INFOS = 529
    LOCAL_QUEST_STARTED = 530
    LOCAL_QUEST_ACTIVE = 531
    LOCAL_QUEST_INACTIVE = 532
    LOCAL_QUEST_PROGRESS_UPDATE = 533
    NEW_UNRESTRICTED_PVP_ZONE = 534
    TEMPORARY_FLAGGING_STATUS_UPDATE = 535
    SPELL_TEST_PERFORMANCE_UPDATE = 536
    TRANSFORMATION = 537
    TRANSFORMATION_END = 538
    UPDATE_TRUSTLEVEL = 539
    REVEAL_HIDDEN_TIME_STAMPS = 540
    MODIFY_ITEM_TRAIT_FINISHED = 541
    REROLL_ITEM_TRAIT_VALUE_FINISHED = 542
    HUNT_QUEST_PROGRESS_INFO = 543
    HUNT_STARTED = 544
    HUNT_FINISHED = 545
    HUNT_ABORTED = 546
    HUNT_MISSION_STEP_STATE_UPDATE = 547
    NEW_HUNT_TRACK = 548
    HUNT_MISSION_UPDATE = 549
    HUNT_QUEST_MISSION_PROGRESS_UPDATE = 550
    HUNT_TRACK_USED = 551
    HUNT_TRACK_USEABLE_AGAIN = 552
    MINIMAP_HUNT_TRACK_MARKERS = 553
    NO_TRACKS_FOUND = 554
    HUNT_QUEST_ABORTED = 555
    INTERACT_WITH_TRACK_START = 556
    INTERACT_WITH_TRACK_CANCEL = 557
    INTERACT_WITH_TRACK_FINISHED = 558
    NEW_DYNAMIC_COMPOUND = 559
    LEGENDARY_ITEM_DESTROYED = 560
    ATTUNEMENT_INFO = 561
    TERRITORY_CLAIM_RAIDED_RAW_ENERGY_CRYSTAL_RESULT = 562
    CARRIED_OBJECT_EXPIRY_WARNING = 563
    CARRIED_OBJECT_EXPIRED = 564
    TERRITORY_RAID_START = 565
    TERRITORY_RAID_CANCEL = 566
    TERRITORY_RAID_FINISHED = 567
    TERRITORY_RAID_RESULT = 568
    TERRITORY_MONOLITH_ACTIVE_RAID_STATUS = 569
    TERRITORY_MONOLITH_ACTIVE_RAID_CANCELLED = 570
    MONOLITH_ENERGY_STORAGE_UPDATE = 571
    MONOLITH_NEXT_SCHEDULED_OPEN_WORLD_ATTACK_UPDATE = 572
    MONOLITH_PROTECTED_BUILDINGS_DAMAGE_REDUCTION_UPDATE = 573
    NEW_BUILDING_BASE_EVENT = 574
    NEW_FORTIFICATION_BUILDING = 575
    NEW_CASTLE_GATE_BUILDING = 576
    BUILDING_DURABILITY_UPDATE = 577
    MONOLITH_FORTIFICATION_POINTS_UPDATE = 578
    FORTIFICATION_BUILDING_UPGRADE_INFO = 579
    FORTIFICATION_BUILDINGS_DAMAGE_STATE_UPDATE = 580
    SIEGE_NOTIFICATION_EVENT = 581
    UPDATE_ENEMY_WAR_BANNER_ACTIVE = 582
    TERRITORY_ANNOUNCE_PLAYER_EJECTION = 583
    CASTLE_GATE_SWITCH_USE_STARTED = 584
    CASTLE_GATE_SWITCH_USE_FINISHED = 585
    FORTIFICATION_BUILDING_WILL_DOWNGRADE = 586
    BOT_COMMAND = 587
    JOURNAL_ACHIEVEMENT_PROGRESS_UPDATE = 588
    JOURNAL_CLAIMABLE_REWARD_UPDATE = 589
    KEY_SYNC = 590
    LOCAL_QUEST_AREA_GONE = 591
    DYNAMIC_TEMPLATE = 592
    DYNAMIC_TEMPLATE_FORCED_STATE_CHANGE = 593
    NEW_OUTLANDS_TELEPORTATION_PORTAL = 594
    NEW_OUTLANDS_TELEPORTATION_RETURN_PORTAL = 595
    OUTLANDS_TELEPORTATION_BINDING_CLEARED = 596
    OUTLANDS_TELEPORTATION_RETURN_PORTAL_UPDATE_EVENT = 597
    PLAYER_USED_OUTLANDS_TELEPORTATION_PORTAL = 598



from albibong.classes.world_data import WorldData


def handle_operation_move(world_data: WorldData, parameters):
    if not world_data.me:
        return

    world_data.me.update_coords(parameters)



import asyncio
import json
import queue
import threading
from datetime import datetime, timedelta

import websockets

from albibong.classes.dungeon import Dungeon
from albibong.classes.location import Island
from albibong.classes.logger import Logger

logger = Logger(__name__, stdout=True, log_to_file=False)
logger_file = Logger(__name__)


class WebsocketServer(threading.Thread):
    def __init__(self, name, in_queue) -> None:
        super().__init__()
        self.name = name
        self.in_queue = in_queue
        self.stop_event = threading.Event()
        self.connections = set()

    async def handler(self, websocket):
        from albibong.classes.world_data import get_world_data

        self.connections.add(websocket)
        try:
            world_data = get_world_data()
            me = world_data.me
            event_init_world = {
                "type": "init_world",
                "payload": {
                    "me": {
                        "username": me.username,
                        "fame": me.fame_gained,
                        "re_spec": me.re_spec_gained,
                        "silver": me.silver_gained,
                        "might": me.might_gained,
                        "favor": me.favor_gained,
                    },
                    "world": {
                        "map": (
                            world_data.current_map.name
                            if world_data.current_map
                            else "zone in to other map to initialize"
                        ),
                        "dungeon": (
                            world_data.current_dungeon.name
                            if world_data.current_dungeon
                            else "zone in to other map to initialize"
                        ),
                        "isDPSMeterRunning": world_data.is_dps_meter_running,
                    },
                },
            }
            await websocket.send(json.dumps(event_init_world))
            event_init_dungeon_list = {
                "type": "update_dungeon",
                "payload": {"list_dungeon": Dungeon.get_all_dungeon()},
            }
            await websocket.send(json.dumps(event_init_dungeon_list))
            event_init_island_list = {
                "type": "update_island",
                "payload": {"list_island": Island.get_all_island()},
            }
            await websocket.send(json.dumps(event_init_island_list))
            total_harvest_reply = {
                "type": "update_total_harvest_by_date",
                "payload": Island.get_total_harvest_by_date(),
            }
            await websocket.send(json.dumps(total_harvest_reply))
            async for message in websocket:
                event = json.loads(message)
                if event["type"] == "update_is_dps_meter_running":
                    world_data.is_dps_meter_running = event["payload"]["value"]
                    reply = {
                        "type": "update_is_dps_meter_running",
                        "payload": {"value": world_data.is_dps_meter_running},
                    }
                    logger_file.log_dps_meter_state(world_data.is_dps_meter_running)
                    await websocket.send(json.dumps(reply))
                elif event["type"] == "reset_dps_meter":
                    party_members = event["payload"]["party_members"]
                    for member in party_members:
                        char = world_data.characters[member]
                        char.damage_dealt = 0
                        char.healing_dealt = 0
                        char.total_combat_duration = timedelta(0, 0)
                    reply = {
                        "type": "update_damage_meter",
                        "payload": {
                            "party_members": world_data.serialize_party_members()
                        },
                    }
                    await websocket.send(json.dumps(reply))
                elif event["type"] == "reset_player_stats":
                    world_data.me.fame_gained = 0
                    world_data.me.re_spec_gained = 0
                    world_data.me.silver_gained = 0
                    world_data.me.might_gained = 0
                    world_data.me.favor_gained = 0

                    fame = {
                        "type": "update_fame",
                        "payload": {
                            "username": world_data.me.username,
                            "fame_gained": world_data.me.fame_gained,
                        },
                    }
                    re_spec = {
                        "type": "update_re_spec",
                        "payload": {
                            "username": world_data.me.username,
                            "re_spec_gained": world_data.me.re_spec_gained,
                        },
                    }
                    silver = {
                        "type": "update_silver",
                        "payload": {
                            "username": world_data.me.username,
                            "silver_gained": world_data.me.silver_gained,
                        },
                    }
                    might_and_favor = {
                        "type": "update_might_and_favor",
                        "payload": {
                            "username": world_data.me.username,
                            "favor_gained": world_data.me.favor_gained,
                            "might_gained": world_data.me.might_gained,
                        },
                    }
                    await websocket.send(json.dumps(fame))
                    await websocket.send(json.dumps(re_spec))
                    await websocket.send(json.dumps(silver))
                    await websocket.send(json.dumps(might_and_favor))

                elif event["type"] == "refresh_dungeon_list":
                    reply = {
                        "type": "update_dungeon",
                        "payload": {"list_dungeon": Dungeon.get_all_dungeon()},
                    }
                    await websocket.send(json.dumps(reply))
                elif event["type"] == "update_dungeon_tier":
                    id = event["payload"]["id"]
                    value = event["payload"]["value"]
                    dungeon: Dungeon = Dungeon.update(tier=value).where(
                        Dungeon.id == id
                    )
                    dungeon.execute()
                    reply = {
                        "type": "update_dungeon",
                        "payload": {"list_dungeon": Dungeon.get_all_dungeon()},
                    }
                    await websocket.send(json.dumps(reply))
                elif event["type"] == "update_dungeon_name":
                    id = event["payload"]["id"]
                    value = event["payload"]["value"]
                    dungeon: Dungeon = Dungeon.update(name=value).where(
                        Dungeon.id == id
                    )
                    dungeon.execute()
                    reply = {
                        "type": "update_dungeon",
                        "payload": {"list_dungeon": Dungeon.get_all_dungeon()},
                    }
                    await websocket.send(json.dumps(reply))
                elif event["type"] == "refresh_island_list":
                    refresh_island_reply = {
                        "type": "update_island",
                        "payload": {"list_island": Island.get_all_island()},
                    }
                    total_harvest_reply = {
                        "type": "update_total_harvest_by_date",
                        "payload": Island.get_total_harvest_by_date(),
                    }
                    await websocket.send(json.dumps(refresh_island_reply))
                    await websocket.send(json.dumps(total_harvest_reply))
                elif event["type"] == "update_total_harvested_date_range":
                    date = event["payload"]["date"]
                    format = "%Y-%m-%d"
                    total_harvest_reply = {
                        "type": "update_total_harvest_by_date",
                        "payload": Island.get_total_harvest_by_date(
                            date=datetime.strptime(date, format)
                        ),
                    }
                    await websocket.send(json.dumps(total_harvest_reply))

            event = {}
            send_event(event)
        finally:
            self.connections.remove(websocket)

    async def main(self):
        async with websockets.serve(self.handler, "", 8081):
            while True:
                if self.stop_event.is_set():
                    return

                while not self.in_queue.empty():
                    if self.stop_event.is_set():
                        return

                    event = self.in_queue.get()
                    if len(self.connections) > 0:
                        # logger.info(f"broadcast {event}")
                        websockets.broadcast(self.connections, json.dumps(event))
                        await asyncio.sleep(0)

                await asyncio.sleep(0)

    def run(self):
        logger.info(f"Thread {self.name} started")

        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        loop.run_until_complete(self.main())

    def stop(self):
        logger.info(f"Thread {self.name} stopped")
        self.stop_event.set()


event_queue = queue.Queue()
ws_server = WebsocketServer(name="ws_server", in_queue=event_queue)


def send_event(event):
    event_queue.put(event)


def get_ws_server():
    return ws_server



import os
import platform
from importlib.metadata import version
from uuid import UUID

from scapy.all import conf, get_if_addr

FILENAME = os.path.join(os.path.expanduser("~"), "Albibong/Debug/user_spec.txt")
os.makedirs(os.path.dirname(FILENAME), exist_ok=True)


class Utils:

    @staticmethod
    def convert_int_arr_to_uuid(arr):
        b = bytes(arr)
        return UUID(bytes=b)

    @staticmethod
    def get_user_specifications(install_source: str):
        os_ver = f"{platform.system()} {platform.release()}"
        python_ver = platform.python_version()
        albibong_ver = version("albibong")
        interfaces = f"{conf.iface} ({get_if_addr(conf.iface)})"

        text = f"\n===\n\nAlbibong Ver        : {albibong_ver}\nPython Ver          : {python_ver}\nOS Ver              : {os_ver}\nInstallation Source : {install_source}\nInterfaces          : {interfaces}\n\n===\n"
        print(text)
        with open(FILENAME, "w+") as file:
            file.write(text)



from albibong.classes.world_data import WorldData
from albibong.threads.websocket_server import send_event


def handle_event_might_and_favor_received_event(world_data: WorldData, parameters):
    world_data.me.update_might_and_favor(parameters)
    event = {
        "type": "update_might_and_favor",
        "payload": {
            "username": world_data.me.username,
            "favor_gained": world_data.me.favor_gained,
            "might_gained": world_data.me.might_gained,
        },
    }
    send_event(event)

    if world_data.current_dungeon:
        world_data.current_dungeon.update_might_and_favor(parameters)



import json
import os
import queue
import random
import socket
import sys
import uuid
from datetime import datetime, timedelta
from pathlib import Path
from time import sleep

import webview
from peewee_migrate import Router
from scapy.all import rdpcap

from albibong.classes.logger import Logger
from albibong.classes.packet_handler import PacketHandler
from albibong.classes.utils import Utils
from albibong.models.models import SQLITE_DB, db
from albibong.threads.http_server import HttpServerThread
from albibong.threads.packet_handler_thread import PacketHandlerThread
from albibong.threads.sniffer_thread import SnifferThread
from albibong.threads.websocket_server import get_ws_server

logger = Logger(__name__, stdout=True, log_to_file=False)
PORT = random.randrange(8500, 8999)

home_dir = os.path.expanduser("~")
DUNGEON_JSON = f"{home_dir}/Albibong/list_dungeon.json"

current_path = os.path.dirname(__file__)
MIGRATION_FOLDER = os.path.join(current_path, f"migrations")


def read_pcap(path):
    packet_handler = PacketHandler()
    scapy_cap = rdpcap(path)
    for packet in scapy_cap:
        packet_handler.handle(packet)


def get_dungeon_end_time(str, start_time):
    timer = str.split(":")
    seconds = int(timer[0]) * 3600 + int(timer[1]) * 60 + int(timer[2])
    return timedelta(seconds=seconds) + start_time


def check_db_validity():
    migrate = Router(db, migrate_dir=MIGRATION_FOLDER)

    if Path(SQLITE_DB).is_file() == False:
        if Path(DUNGEON_JSON).is_file() == True:

            migrate.run("001_init")

            # convert json db to sqlite db
            json_data = json.load(open(DUNGEON_JSON))

            for dungeon in json_data:
                start_time = datetime.strptime(
                    dungeon["date_time"], "%a %d %b %Y, %I:%M%p"
                )
                type_splitted = set(dungeon["type"].split("_"))

                if "CORRUPTED" in type_splitted:
                    converted_type = "CORRUPTED DUNGEON"
                elif "HARDCORE" in type_splitted:
                    converted_type = "HARDCORE EXPEDITION"
                elif "10V10" in type_splitted:
                    converted_type = "HELLGATE 10V10"
                elif "5V5" in type_splitted:
                    converted_type = "HELLGATE 5V5"
                elif "2V2" in type_splitted:
                    converted_type = "HELLGATE 2V2"
                elif "DUNGEON" in type_splitted:
                    converted_type = "STATIC DUNGEON"
                elif "RANDOMDUNGEON" in type_splitted:
                    converted_type = "SPAWN DUNGEON"

                asd = [
                    ("id", str(uuid.uuid4())),
                    ("type", f"{converted_type}"),
                    ("name", f'{dungeon["name"]}'),
                    ("tier", f'{dungeon["tier"]}'),
                    ("fame", f'{dungeon["fame"]}'),
                    ("silver", f'{dungeon["silver"]}'),
                    ("re_spec", f'{dungeon["re_spec"]}'),
                    ("start_time", f"{start_time}"),
                    (
                        "end_time",
                        f'{get_dungeon_end_time(dungeon["time_elapsed"], start_time)}',
                    ),
                    ("meter", ""),
                ]

                columns = ", ".join([x[0] for x in asd])
                values = ", ".join([f'"{x[1]}"' for x in asd])
                sql_query = f"INSERT INTO dungeon ({columns}) VALUES ({values})"

                db.execute_sql(sql_query)

    migrate.run()


def sniff(useWebview, is_debug=False):

    check_db_validity()

    _sentinel = object()
    packet_queue = queue.Queue()

    p = SnifferThread(
        name="sniffer", out_queue=packet_queue, sentinel=_sentinel, is_debug=is_debug
    )
    c = PacketHandlerThread(
        name="packet_handler",
        in_queue=packet_queue,
        sentinel=_sentinel,
    )

    p.start()
    c.start()

    ws_server = get_ws_server()
    ws_server.start()

    if useWebview:
        sock = socket.socket()
        sock.bind(("", 0))
        port = sock.getsockname()[1]
        sock.close()

        http_server = HttpServerThread(name="http_server", port=port)
        http_server.start()

        window = webview.create_window(
            "Albibong",
            url=f"http://localhost:{port}/",
            width=1280,
            height=720,
            zoomable=True,
        )

        def on_closing():
            c.stop()
            p.stop()
            ws_server.stop()
            http_server.stop()
            db.close()

        window.events.closing += on_closing

        webview.start()
    else:
        try:
            while True:
                sleep(100)
        except KeyboardInterrupt as e:
            p.stop()
            ws_server.stop()


def main(useWebview=True):
    if len(sys.argv) > 1:
        if sys.argv[-1] == "--debug":
            Utils.get_user_specifications("pip")
            sniff(useWebview, is_debug=True)
        else:
            ws_server = get_ws_server()
            ws_server.start()
            read_pcap(sys.argv[1])
    else:
        sniff(useWebview)



from uuid import UUID

from albibong.classes.character import Character
from albibong.classes.coords import Coords
from albibong.classes.dungeon import Dungeon
from albibong.classes.item import Item
from albibong.classes.location import Location


class WorldData:

    def __init__(self) -> None:
        self.me: Character = Character(
            id=None,
            uuid=None,
            username="not initialized",
            guild="not initialized",
            alliance="not initialized",
            coords=Coords(0, 0),
        )
        self.current_map: Location = None
        self.current_dungeon: Dungeon = None
        self.characters: dict[str, Character] = {}
        self.char_id_to_username: dict[int, str] = {self.me.id: self.me.username}
        self.char_uuid_to_username: dict[UUID, str] = {self.me.uuid: self.me.username}
        self.change_equipment_log: dict[int, list] = {}
        self.party_members: set[str] = set()
        self.is_dps_meter_running: bool = True

    def serialize_party_members(self):
        serialized = []

        total_damage = 0
        total_heal = 0

        # get total damage and heal for percentage
        for key, value in self.characters.items():
            if key in self.party_members:
                total_damage += value.damage_dealt
                total_heal += value.healing_dealt

        for member in self.party_members:
            # member character initialized
            if member in self.characters:
                char = self.characters[member]
                username = char.username
                damage_dealt = char.damage_dealt
                damage_percent = (
                    round(char.damage_dealt / total_damage * 100, 2)
                    if total_damage > 0
                    else 0
                )
                healing_dealt = char.healing_dealt
                heal_percent = (
                    round(char.healing_dealt / total_heal * 100, 2)
                    if total_heal > 0
                    else 0
                )
                duration = char.total_combat_duration

                combat_duration = str(duration).split(".")[0]
                dps = (
                    damage_dealt // duration.total_seconds()
                    if duration.total_seconds() != 0
                    else 0
                )
                weapon = (
                    Item.serialize(char.equipment[0])["image"]
                    if char.equipment != []
                    else "/No Equipment.png"
                )

            # member character not initialized
            else:
                username = member
                damage_dealt = 0
                damage_percent = 0
                healing_dealt = 0
                heal_percent = 0
                combat_duration = 0
                dps = 0
                weapon = "/No Equipment.png"

            data = {
                "username": username,
                "damage_dealt": damage_dealt,
                "damage_percent": damage_percent,
                "healing_dealt": healing_dealt,
                "heal_percent": heal_percent,
                "combat_duration": combat_duration,
                "dps": dps,
                "weapon": weapon,
            }
            serialized.append(data)
        serialized.sort(key=lambda x: x["damage_dealt"], reverse=True)
        return serialized


world_data = WorldData()


def get_world_data():
    return world_data



import sys

from albibong import main
from albibong.classes.logger import use_logger
from albibong.classes.utils import Utils

if __name__ == "__main__":
    use_logger(True)

    if sys.argv[-1] == "--debug":
        Utils.get_user_specifications("source code")
    main(useWebview=False)



from albibong.classes.character import Character
from albibong.classes.world_data import WorldData


def handle_event_in_combat_state_update(world_data: WorldData, parameters):

    if parameters[0] not in world_data.char_id_to_username:
        # character not initialized
        return

    name = world_data.char_id_to_username[parameters[0]]
    char: Character = world_data.characters[name]

    if 1 in parameters:
        char.update_combat_duration(True)
    else:
        char.update_combat_duration(False)



from uuid import UUID

from albibong.classes.event_handler.world_data_utils import WorldDataUtils
from albibong.classes.world_data import WorldData


def handle_event_party_joined(world_data: WorldData, parameters):
    world_data.party_members = set(parameters[5])
    WorldDataUtils.ws_update_damage_meter(world_data)


def handle_event_party_disbanded(world_data: WorldData, parameters):
    world_data.party_members = {world_data.me.username}
    WorldDataUtils.ws_update_damage_meter(world_data)


def handle_event_party_player_joined(world_data: WorldData, parameters):
    world_data.party_members.add(parameters[2])
    WorldDataUtils.ws_update_damage_meter(world_data)


def handle_event_party_player_left(world_data: WorldData, parameters):
    uuid = UUID(bytes=bytes(parameters[1]))

    if uuid in world_data.char_uuid_to_username:
        name = world_data.char_uuid_to_username[uuid]

        # self out party
        if name == world_data.me.username:
            world_data.party_members = {world_data.me.username}
            WorldDataUtils.ws_update_damage_meter(world_data)



import uuid
from datetime import datetime

from peewee import *
from playhouse.sqlite_ext import JSONField

from albibong.models.models import BaseModel

DIVISOR = 10000


class Dungeon(BaseModel):

    id = UUIDField(unique=True, default=uuid.uuid4)
    type = CharField()
    name = CharField()
    tier = FloatField(default=0)
    fame = FloatField(default=0)
    silver = FloatField(default=0)
    re_spec = FloatField(default=0)
    might = FloatField(default=0)
    favor = FloatField(default=0)
    start_time = DateTimeField(default=datetime.now)
    end_time = DateTimeField(null=True, default=None)
    meter = JSONField(default=dict)

    @staticmethod
    def serialize(dungeon):
        time_elapsed = dungeon.end_time - dungeon.start_time
        total_hours = time_elapsed.total_seconds() / 3600

        return {
            "id": str(dungeon.id),
            "type": dungeon.type,
            "name": dungeon.name,
            "tier": dungeon.tier,
            "fame": dungeon.fame,
            "fame_per_hour": dungeon.fame / total_hours,
            "silver": dungeon.silver,
            "silver_per_hour": dungeon.silver / total_hours,
            "re_spec": dungeon.re_spec,
            "re_spec_per_hour": dungeon.re_spec / total_hours,
            "might": dungeon.might,
            "might_per_hour": dungeon.might / total_hours,
            "favor": dungeon.favor,
            "favor_per_hour": dungeon.favor / total_hours,
            "date_time": dungeon.start_time.strftime("%a %d %b %Y, %I:%M%p"),
            "time_elapsed": (
                str(time_elapsed).split(".")[0] if dungeon.end_time else ""
            ),
            "meter": dungeon.meter,
        }

    def set_end_time(self):
        self.end_time = datetime.now()

    def update_fame(self, parameters):
        fame = parameters[2] / DIVISOR
        self.fame += fame

    def update_re_spec(self, parameters):
        re_spec = parameters[2] / DIVISOR
        self.re_spec += re_spec

    def update_might_and_favor(self, parameters):
        self.favor += parameters[4] / DIVISOR
        self.might += parameters[1] / DIVISOR

    def update_loot(self, parameters):
        if 3 in parameters and parameters[3] == True:
            silver = parameters[5] / DIVISOR
            self.silver += silver

    def update_meter(self, party):
        self.meter = party

    @staticmethod
    def get_all_dungeon():
        query = Dungeon.select().order_by(Dungeon.start_time.desc())
        return [Dungeon.serialize(dungeon) for dungeon in query]



from albibong.classes.character import Character
from albibong.classes.dungeon import Dungeon
from albibong.classes.location import Island, Location
from albibong.classes.world_data import WorldData
from albibong.threads.websocket_server import send_event


class WorldDataUtils:

    def save_current_island(current_map: Island):
        current_map.save(force_insert=True)
        WorldDataUtils.ws_update_total_harvest_by_date(
            Island.get_total_harvest_by_date()
        )
        WorldDataUtils.ws_update_island(Island.get_all_island())

    def set_island_status(current_island: Island, parameters):
        check_map = (
            Location.get_location_from_code(parameters[1])
            if 1 in parameters
            else Location.get_location_from_code(parameters[0])
        )
        check_map_name = (
            f"{parameters[2]}'s {check_map.name}" if 2 in parameters else check_map.name
        )
        if current_island.name != check_map_name:
            if current_island.crops != {} or current_island.animals != {}:
                WorldDataUtils.save_current_island(current_island)

    def ws_update_total_harvest_by_date(payload):
        event = {
            "type": "update_total_harvest_by_date",
            "payload": payload,
        }
        send_event(event)

    def ws_update_island(list_island: list):
        event = {
            "type": "update_island",
            "payload": {"list_island": list_island},
        }
        send_event(event)

    def end_current_dungeon(world_data: WorldData):
        if world_data.current_dungeon:
            world_data.current_dungeon.set_end_time()
            world_data.current_dungeon.update_meter(
                world_data.serialize_party_members()
            )
            world_data.current_dungeon.save(force_insert=True)
            WorldDataUtils.ws_update_dungeon(Dungeon.get_all_dungeon())
            world_data.current_dungeon = None

    def ws_update_dungeon(list_dungeon: list):
        event = {
            "type": "update_dungeon",
            "payload": {"list_dungeon": list_dungeon},
        }
        send_event(event)

    @staticmethod
    def start_current_dungeon(world_data: WorldData, type: str, name: str):
        if world_data.current_dungeon == None:
            new_dungeon = Dungeon(type=type, name=name)
            world_data.current_dungeon = new_dungeon

    @staticmethod
    def set_dungeon_status(
        world_data: WorldData, check_map: Location, map_type_splitted: set
    ):
        if "EXPEDITION" in map_type_splitted or "HELLGATE" in map_type_splitted:
            WorldDataUtils.start_current_dungeon(
                world_data, type=check_map.type, name=check_map.name
            )
        elif "DUNGEON" in map_type_splitted:
            WorldDataUtils.start_current_dungeon(
                world_data,
                type=check_map.type,
                name=(
                    f"{check_map.name} at {world_data.current_map.name}"
                    if world_data.current_map
                    else check_map.name
                ),
            )
        elif (
            "EXPEDITION" not in map_type_splitted or "DUNGEON" not in map_type_splitted
        ):
            WorldDataUtils.end_current_dungeon(world_data)
            return False

    @staticmethod
    def convert_id_to_name(world_data: WorldData, old_id, new_id, char: Character):
        if old_id in world_data.char_id_to_username:
            world_data.char_id_to_username.pop(old_id)  # delete old relative id
        char.id = new_id
        world_data.char_id_to_username[char.id] = char.username  # add new relative id

    @staticmethod
    def update_damage_or_heal(world_data: WorldData, target, inflictor, nominal):

        if inflictor not in world_data.char_id_to_username:
            # character not initialized yet
            return

        username = world_data.char_id_to_username[inflictor]

        if username == "not initialized":
            # self not initialized
            return

        char: Character = world_data.characters[username]

        if nominal < 0:
            if target == inflictor:
                # suicide
                return
            char.update_damage_dealt(abs(nominal))
        else:
            char.update_heal_dealt(nominal)

        WorldDataUtils.ws_update_damage_meter(world_data)

    def ws_update_damage_meter(world_data: WorldData):
        event = {
            "type": "update_damage_meter",
            "payload": {"party_members": world_data.serialize_party_members()},
        }
        send_event(event)

    def ws_update_location(world_data: WorldData):
        event = {
            "type": "update_location",
            "payload": {
                "map": (
                    world_data.current_map.name if world_data.current_map else "None"
                ),
                "dungeon": (
                    world_data.current_dungeon.name
                    if world_data.current_dungeon
                    else "None"
                ),
            },
        }
        send_event(event)



import os
import threading
from time import sleep

from scapy.all import AsyncSniffer, wrpcapng

from albibong.classes.logger import Logger
from albibong.threads.websocket_server import send_event

home_dir = os.path.expanduser("~")
pcap_file = home_dir + "/Albibong/Debug/debug.pcapng"

logger = Logger(__name__, stdout=True, log_to_file=False)


class SnifferThread(threading.Thread):
    def __init__(self, name, out_queue, sentinel, is_debug=False):
        super().__init__()
        self.name = name
        self.out_queue = out_queue
        self.sentinel = sentinel
        self.is_debug = is_debug
        self.sniffer = AsyncSniffer(filter="udp and port 5056", prn=self.push_packet)
        self.packet_counter = 0
        self.timer_exit = threading.Event()
        self.all_packets = []

    def push_packet(self, packet):
        self.out_queue.put(packet)
        self.all_packets.append(packet)
        self.packet_counter += 1

    def run(self):
        logger.info(f"Thread {self.name} started")
        self.sniffer.start()
        timer = threading.Thread(target=self.start_timer, args=[5], daemon=True)
        timer.run()

    def stop(self):
        logger.info(f"Thread {self.name} stopped")
        self.sniffer.stop()
        self.out_queue.put(self.sentinel)
        self.quit_timer()
        if self.is_debug:
            wrpcapng(pcap_file, self.all_packets)

    def start_timer(self, seconds):
        while not self.timer_exit.is_set():
            self.timer_exit.wait(seconds)

            if self.packet_counter > 0:
                msg = {
                    "type": "health_check",
                    "payload": {"status": "passed", "message": "Passed"},
                }
            else:
                msg = {
                    "type": "health_check",
                    "payload": {"status": "failed", "message": "No Packets Received"},
                }
            send_event(msg)
            self.packet_counter = 0

    def quit_timer(self):
        self.timer_exit.set()



from albibong.classes.character import Character
from albibong.classes.world_data import WorldData
from albibong.threads.websocket_server import send_event


def handle_event_other_grabbed_loot(world_data: WorldData, parameters):

    # update char data
    if 2 in parameters and parameters[2] in world_data.characters:
        char: Character = world_data.characters[parameters[2]]
        char.update_loot(parameters)
        event = {
            "type": "update_silver",
            "payload": {
                "username": char.username,
                "silver_gained": char.silver_gained,
            },
        }
        send_event(event)

    # update dungeon data
    if world_data.current_dungeon and parameters[2] == world_data.me.username:
        world_data.current_dungeon.update_loot(parameters)



import threading
from albibong.classes.logger import Logger
from albibong.classes.packet_handler import PacketHandler

logger = Logger(__name__, stdout=True, log_to_file=False)


class PacketHandlerThread(threading.Thread):
    def __init__(self, name, in_queue, sentinel):
        super().__init__()
        self.name = name
        self.in_queue = in_queue
        self.sentinel = sentinel
        self.packet_handler = PacketHandler()
        self.stop_event = threading.Event()

    def run(self):
        logger.info(f"Thread {self.name} started")
        while True:
            if self.stop_event.is_set():
                break

            packet = self.in_queue.get()
            if packet == self.sentinel:
                logger.info(f"Thread {self.name} stopped")
                break

            self.packet_handler.handle(packet)

    def stop(self):
        logger.info(f"Thread {self.name} stopped")
        self.stop_event.set()



from albibong.classes.event_handler.world_data_utils import WorldDataUtils
from albibong.classes.location import Location
from albibong.classes.utils import Utils
from albibong.classes.world_data import WorldData
from albibong.threads.websocket_server import send_event


def handle_operation_join(world_data: WorldData, parameters):
    # set my character
    world_data.me.username = parameters[2]
    world_data.me.uuid = Utils.convert_int_arr_to_uuid(parameters[1])
    world_data.me.guild = parameters[57] if 57 in parameters else ""
    world_data.me.alliance = parameters[77] if 77 in parameters else ""

    # update relative id if character has initialized before
    WorldDataUtils.convert_id_to_name(
        world_data,
        old_id=world_data.me.id,
        new_id=parameters[0],
        char=world_data.me,
    )

    if world_data.me.id in world_data.change_equipment_log:
        world_data.me.update_equipment(
            world_data.change_equipment_log[world_data.me.id]
        )

    # put self in characters list
    world_data.characters[world_data.me.username] = world_data.me
    world_data.char_uuid_to_username[world_data.me.uuid] = world_data.me.username

    # put self in party
    world_data.party_members.add(world_data.me.username)

    # set map my character is currently in
    if parameters[8][0] == "@":
        area = parameters[8].split("@")
        if area[1] == "RANDOMDUNGEON" or area[1] == "MISTS":
            check_map = Location.get_location_from_code(area[1])
            WorldDataUtils.start_current_dungeon(
                world_data, type=check_map.type, name=check_map.name
            )

    ws_init_character(world_data)
    WorldDataUtils.ws_update_location(world_data)
    WorldDataUtils.ws_update_damage_meter(world_data)


def ws_init_character(world_data: WorldData):
    event = {
        "type": "init_character",
        "payload": {
            "username": world_data.me.username,
            "fame": world_data.me.fame_gained,
            "re_spec": world_data.me.re_spec_gained,
            "silver": world_data.me.silver_gained,
            "might": world_data.me.might_gained,
            "favor": world_data.me.favor_gained,
            "weapon": world_data.me.equipment[0].image,
        },
    }
    send_event(event)



import json
import os
import stat
import uuid
from datetime import date, datetime, timedelta

from peewee import CharField, DateTimeField, UUIDField, fn
from playhouse.sqlite_ext import JSONField

from albibong.classes.dungeon import Dungeon
from albibong.classes.item import Item
from albibong.models.models import BaseModel

mapsJsonPath = os.path.join(os.path.dirname(__file__), "../resources/maps.json")
with open(mapsJsonPath) as json_file:
    map_data = json.load(json_file)


class Location(BaseModel):

    uuid = UUIDField(unique=True, default=uuid.uuid4)
    id = CharField()
    name = CharField()
    type = CharField()

    @classmethod
    def get_location_from_code(cls, code: str):
        location = map_data[code]

        return cls(id=location["id"], name=location["name"], type=location["type"])


class Island(Location, BaseModel):

    uuid = UUIDField(unique=True, default=uuid.uuid4)
    id = CharField()
    name = CharField()
    type = CharField()
    start_time = DateTimeField(default=datetime.now)
    crops = JSONField(default=dict)
    animals = JSONField(default=dict)

    def add_crop(self, unique_name, quantity):
        crop = Item.get_item_from_unique_name(unique_name)

        if crop.name not in self.crops:
            self.crops[crop.name] = Item.serialize(crop)

        self.crops[crop.name]["quantity"] += quantity

    def add_animal(self, unique_name, quantity):
        animal = Item.get_item_from_unique_name(unique_name)

        if animal.name not in self.animals:
            self.animals[animal.name] = Item.serialize(animal)

        self.animals[animal.name]["quantity"] += quantity

    @staticmethod
    def serialize(island):
        data = {
            "id": str(island.uuid),
            "name": island.name,
            "type": island.type,
            "date_time": island.start_time.strftime("%a %d %b %Y, %I:%M%p"),
            "crops": island.serialize_crops(),
            "animals": island.serialize_animals(),
        }
        return data

    def serialize_animals(self):
        serialized = []

        for key, value in self.animals.items():
            serialized.append(value)

        return serialized

    def serialize_crops(self):
        serialized = []

        for key, value in self.crops.items():
            serialized.append(value)

        return serialized

    @staticmethod
    def get_all_island():
        query = Island.select().order_by(Island.start_time.desc())
        a = [Island.serialize(island) for island in query]
        return a

    @staticmethod
    def get_total_harvest_by_date(date=datetime.today()):

        date = date
        tomorrow = date + timedelta(days=1)

        query = Island.select().where(
            Island.start_time >= date.date(),
            Island.start_time < tomorrow.date(),
        )

        total_crops = {}
        total_animals = {}

        for island in query:

            crops = island.crops
            for key, value in crops.items():
                if key not in total_crops:
                    total_crops[key] = {
                        "name": key,
                        "quantity": value["quantity"],
                        "image": value["image"],
                    }
                else:
                    total_crops[key]["quantity"] += value["quantity"]

            animals = island.animals
            for key, value in animals.items():
                if key not in total_animals:
                    total_animals[key] = {
                        "name": key,
                        "quantity": value["quantity"],
                        "image": value["image"],
                    }
                else:
                    total_animals[key]["quantity"] += value["quantity"]

        crops_result = []
        for key, value in total_crops.items():
            crops_result.append(value)

        animals_result = []
        for key, value in total_animals.items():
            animals_result.append(value)

        return {
            "crops": crops_result,
            "animals": animals_result,
            "date": str(date.date()),
        }

    @staticmethod
    def get_island_from_code(code: str):
        island = map_data[code]

        obj = Island(id=island["id"], name=island["name"], type=island["type"])

        return obj



from albibong.classes.world_data import WorldData
from albibong.threads.websocket_server import send_event


def handle_event_update_fame(world_data: WorldData, parameters):
    world_data.me.update_fame(parameters)
    event = {
        "type": "update_fame",
        "payload": {
            "username": world_data.me.username,
            "fame_gained": world_data.me.fame_gained,
        },
    }
    send_event(event)
    if world_data.current_dungeon:
        world_data.current_dungeon.update_fame(parameters)



from enum import Enum


class OperationCode(Enum):
    UNUSED = 0
    PING = 1
    JOIN = 2
    VERSIONED_OPERATION = 3
    CREATE_ACCOUNT = 4
    LOGIN = 5
    CREATE_GUEST_ACCOUNT = 6
    SEND_CRASH_LOG = 7
    SEND_TRACE_ROUTE = 8
    SEND_VFX_STATS = 9
    SEND_GAME_PING_INFO = 10
    CREATE_CHARACTER = 11
    DELETE_CHARACTER = 12
    SELECT_CHARACTER = 13
    ACCEPT_POPUPS = 14
    REDEEM_KEYCODE = 15
    GET_GAME_SERVER_BY_CLUSTER = 16
    GET_SHOP_PURCHASE_URL = 17
    GET_REFERRAL_SEASON_DETAILS = 18
    GET_REFERRAL_LINK = 19
    GET_SHOP_TILES_FOR_CATEGORY = 20
    MOVE = 21
    ATTACK_START = 22
    CAST_START = 23
    CAST_CANCEL = 24
    TERMINATE_TOGGLE_SPELL = 25
    CHANNELING_CANCEL = 26
    ATTACK_BUILDING_START = 27
    INVENTORY_DESTROY_ITEM = 28
    INVENTORY_MOVE_ITEM = 29
    INVENTORY_RECOVER_ITEM = 30
    INVENTORY_RECOVER_ALL_ITEMS = 31
    INVENTORY_SPLIT_STACK = 32
    INVENTORY_SPLIT_STACK_INTO = 33
    GET_CLUSTER_DATA = 34
    CHANGE_CLUSTER = 35
    CONSOLE_COMMAND = 36
    CHAT_MESSAGE = 37
    REPORT_CLIENT_ERROR = 38
    REGISTER_TO_OBJECT = 39
    UN_REGISTER_FROM_OBJECT = 40
    CRAFT_BUILDING_CHANGE_SETTINGS = 41
    CRAFT_BUILDING_TAKE_MONEY = 42
    REPAIR_BUILDING_CHANGE_SETTINGS = 43
    REPAIR_BUILDING_TAKE_MONEY = 44
    ACTION_BUILDING_CHANGE_SETTINGS = 45
    HARVEST_START = 46
    HARVEST_CANCEL = 47
    TAKE_SILVER = 48
    ACTION_ON_BUILDING_START = 49
    ACTION_ON_BUILDING_CANCEL = 50
    INSTALL_RESOURCE_START = 51
    INSTALL_RESOURCE_CANCEL = 52
    INSTALL_SILVER = 53
    BUILDING_FILL_NUTRITION = 54
    BUILDING_CHANGE_RENOVATION_STATE = 55
    BUILDING_BUY_SKIN = 56
    BUILDING_CLAIM = 57
    BUILDING_GIVEUP = 58
    BUILDING_NUTRITION_SILVER_STORAGE_DEPOSIT = 59
    BUILDING_NUTRITION_SILVER_STORAGE_WITHDRAW = 60
    BUILDING_NUTRITION_SILVER_REWARD_SET = 61
    CONSTRUCTION_SITE_CREATE = 62
    PLACEABLE_OBJECT_PLACE = 63
    PLACEABLE_OBJECT_PLACE_CANCEL = 64
    PLACEABLE_OBJECT_PICKUP = 65
    FURNITURE_OBJECT_USE = 66
    FARMABLE_HARVEST = 67
    FARMABLE_FINISH_GROWN_ITEM = 68
    FARMABLE_DESTROY = 69
    FARMABLE_GET_PRODUCT = 70
    FARMABLE_FILL = 71
    TEAR_DOWN_CONSTRUCTION_SITE = 72
    AUCTION_CREATE_OFFER = 73
    AUCTION_CREATE_REQUEST = 74
    AUCTION_GET_OFFERS = 75
    AUCTION_GET_REQUESTS = 76
    AUCTION_BUY_OFFER = 77
    AUCTION_ABORT_AUCTION = 78
    AUCTION_MODIFY_AUCTION = 79
    AUCTION_ABORT_OFFER = 80
    AUCTION_ABORT_REQUEST = 81
    AUCTION_SELL_REQUEST = 82
    AUCTION_GET_FINISHED_AUCTIONS = 83
    AUCTION_GET_FINISHED_AUCTIONS_COUNT = 84
    AUCTION_FETCH_AUCTION = 85
    AUCTION_GET_MY_OPEN_OFFERS = 86
    AUCTION_GET_MY_OPEN_REQUESTS = 87
    AUCTION_GET_MY_OPEN_AUCTIONS = 88
    AUCTION_GET_ITEM_AVERAGE_STATS = 89
    AUCTION_GET_ITEM_AVERAGE_VALUE = 90
    CONTAINER_OPEN = 91
    CONTAINER_CLOSE = 92
    CONTAINER_MANAGE_SUB_CONTAINER = 93
    RESPAWN = 94
    SUICIDE = 95
    JOIN_GUILD = 96
    LEAVE_GUILD = 97
    CREATE_GUILD = 98
    INVITE_TO_GUILD = 99
    DECLINE_GUILD_INVITATION = 100
    KICK_FROM_GUILD = 101
    INSTANT_JOIN_GUILD = 102
    DUELLING_CHALLENGE_PLAYER = 103
    DUELLING_ACCEPT_CHALLENGE = 104
    DUELLING_DENY_CHALLENGE = 105
    CHANGE_CLUSTER_TAX = 106
    CLAIM_TERRITORY = 107
    GIVE_UP_TERRITORY = 108
    CHANGE_TERRITORY_ACCESS_RIGHTS = 109
    GET_MONOLITH_INFO = 110
    GET_CLAIM_INFO = 111
    GET_ATTACK_INFO = 112
    GET_TERRITORY_SEASON_POINTS = 113
    GET_ATTACK_SCHEDULE = 114
    GET_MATCHES = 115
    GET_MATCH_DETAILS = 116
    JOIN_MATCH = 117
    LEAVE_MATCH = 118
    GET_CLUSTER_INSTANCE_INFO_FOR_STATIC_CLUSTER = 119
    CHANGE_CHAT_SETTINGS = 120
    LOGOUT_START = 121
    LOGOUT_CANCEL = 122
    CLAIM_ORB_START = 123
    CLAIM_ORB_CANCEL = 124
    MATCH_LOOT_CHEST_OPENING_START = 125
    MATCH_LOOT_CHEST_OPENING_CANCEL = 126
    DEPOSIT_TO_GUILD_ACCOUNT = 127
    WITHDRAWAL_FROM_ACCOUNT = 128
    CHANGE_GUILD_PAY_UPKEEP_FLAG = 129
    CHANGE_GUILD_TAX = 130
    GET_MY_TERRITORIES = 131
    MORGANA_COMMAND = 132
    GET_SERVER_INFO = 133
    SUBSCRIBE_TO_CLUSTER = 134
    ANSWER_MERCENARY_INVITATION = 135
    GET_CHARACTER_EQUIPMENT = 136
    GET_CHARACTER_STEAM_ACHIEVEMENTS = 137
    GET_CHARACTER_STATS = 138
    GET_KILL_HISTORY_DETAILS = 139
    LEARN_MASTERY_LEVEL = 140
    RE_SPEC_ACHIEVEMENT = 141
    CHANGE_AVATAR = 142
    GET_RANKINGS = 143
    GET_RANK = 144
    GET_GVG_SEASON_RANKINGS = 145
    GET_GVG_SEASON_RANK = 146
    GET_GVG_SEASON_HISTORY_RANKINGS = 147
    GET_GVG_SEASON_GUILD_MEMBER_HISTORY = 148
    KICK_FROM_GV_G_MATCH = 149
    GET_CRYSTAL_LEAGUE_DAILY_SEASON_POINTS = 150
    GET_CHEST_LOGS = 151
    GET_ACCESS_RIGHT_LOGS = 152
    GET_GUILD_ACCOUNT_LOGS = 153
    GET_GUILD_ACCOUNT_LOGS_LARGE_AMOUNT = 154
    INVITE_TO_PLAYER_TRADE = 155
    PLAYER_TRADE_CANCEL = 156
    PLAYER_TRADE_INVITATION_ACCEPT = 157
    PLAYER_TRADE_ADD_ITEM = 158
    PLAYER_TRADE_REMOVE_ITEM = 159
    PLAYER_TRADE_ACCEPT_TRADE = 160
    PLAYER_TRADE_SET_SILVER_OR_GOLD = 161
    SEND_MINI_MAP_PING = 162
    STUCK = 163
    BUY_REAL_ESTATE = 164
    CLAIM_REAL_ESTATE = 165
    GIVE_UP_REAL_ESTATE = 166
    CHANGE_REAL_ESTATE_OUTLINE = 167
    GET_MAIL_INFOS = 168
    GET_MAIL_COUNT = 169
    READ_MAIL = 170
    SEND_NEW_MAIL = 171
    DELETE_MAIL = 172
    MARK_MAIL_UNREAD = 173
    CLAIM_ATTACHMENT_FROM_MAIL = 174
    APPLY_TO_GUILD = 175
    ANSWER_GUILD_APPLICATION = 176
    REQUEST_GUILD_FINDER_FILTERED_LIST = 177
    UPDATE_GUILD_RECRUITMENT_INFO = 178
    REQUEST_GUILD_RECRUITMENT_INFO = 179
    REQUEST_GUILD_FINDER_NAME_SEARCH = 180
    REQUEST_GUILD_FINDER_RECOMMENDED_LIST = 181
    REGISTER_CHAT_PEER = 182
    SEND_CHAT_MESSAGE = 183
    SEND_MODERATOR_MESSAGE = 184
    JOIN_CHAT_CHANNEL = 185
    LEAVE_CHAT_CHANNEL = 186
    SEND_WHISPER_MESSAGE = 187
    SAY = 188
    PLAY_EMOTE = 189
    STOP_EMOTE = 190
    GET_CLUSTER_MAP_INFO = 191
    ACCESS_RIGHTS_CHANGE_SETTINGS = 192
    MOUNT = 193
    MOUNT_CANCEL = 194
    BUY_JOURNEY = 195
    SET_SALE_STATUS_FOR_ESTATE = 196
    RESOLVE_GUILD_OR_PLAYER_NAME = 197
    GET_RESPAWN_INFOS = 198
    MAKE_HOME = 199
    LEAVE_HOME = 200
    RESURRECTION_REPLY = 201
    ALLIANCE_CREATE = 202
    ALLIANCE_DISBAND = 203
    ALLIANCE_GET_MEMBER_INFOS = 204
    ALLIANCE_INVITE = 205
    ALLIANCE_ANSWER_INVITATION = 206
    ALLIANCE_CANCEL_INVITATION = 207
    ALLIANCE_KICK_GUILD = 208
    ALLIANCE_LEAVE = 209
    ALLIANCE_CHANGE_GOLD_PAYMENT_FLAG = 210
    ALLIANCE_GET_DETAIL_INFO = 211
    GET_ISLAND_INFOS = 212
    BUY_MY_ISLAND = 213
    BUY_GUILD_ISLAND = 214
    UPGRADE_MY_ISLAND = 215
    UPGRADE_GUILD_ISLAND = 216
    TERRITORY_FILL_NUTRITION = 217
    TELEPORT_BACK = 218
    PARTY_INVITE_PLAYER = 219
    PARTY_REQUEST_JOIN = 220
    PARTY_ANSWER_INVITATION = 221
    PARTY_ANSWER_JOIN_REQUEST = 222
    PARTY_LEAVE = 223
    PARTY_KICK_PLAYER = 224
    PARTY_MAKE_LEADER = 225
    PARTY_CHANGE_LOOT_SETTING = 226
    PARTY_MARK_OBJECT = 227
    PARTY_SET_ROLE = 228
    SET_GUILD_CODEX = 229
    EXIT_ENTER_START = 230
    EXIT_ENTER_CANCEL = 231
    QUEST_GIVER_REQUEST = 232
    GOLD_MARKET_GET_BUY_OFFER = 233
    GOLD_MARKET_GET_BUY_OFFER_FROM_SILVER = 234
    GOLD_MARKET_GET_SELL_OFFER = 235
    GOLD_MARKET_GET_SELL_OFFER_FROM_SILVER = 236
    GOLD_MARKET_BUY_GOLD = 237
    GOLD_MARKET_SELL_GOLD = 238
    GOLD_MARKET_CREATE_SELL_ORDER = 239
    GOLD_MARKET_CREATE_BUY_ORDER = 240
    GOLD_MARKET_GET_INFOS = 241
    GOLD_MARKET_CANCEL_ORDER = 242
    GOLD_MARKET_GET_AVERAGE_INFO = 243
    TREASURE_CHEST_USING_START = 244
    TREASURE_CHEST_USING_CANCEL = 245
    USE_LOOT_CHEST = 246
    USE_SHRINE = 247
    USE_HELLGATE_SHRINE = 248
    GET_SIEGE_BANNER_INFO = 249
    LABORER_START_JOB = 250
    LABORER_TAKE_JOB_LOOT = 251
    LABORER_DISMISS = 252
    LABORER_MOVE = 253
    LABORER_BUY_ITEM = 254
    LABORER_UPGRADE = 255
    BUY_PREMIUM = 256
    REAL_ESTATE_GET_AUCTION_DATA = 257
    REAL_ESTATE_BID_ON_AUCTION = 258
    FRIEND_INVITE = 259
    FRIEND_ANSWER_INVITATION = 260
    FRIEND_CANCELNVITATION = 261
    FRIEND_REMOVE = 262
    INVENTORY_STACK = 263
    INVENTORY_REORDER = 264
    INVENTORY_DROP_ALL = 265
    INVENTORY_ADD_TO_STACKS = 266
    EQUIPMENT_ITEM_CHANGE_SPELL = 267
    EXPEDITION_REGISTER = 268
    EXPEDITION_REGISTER_CANCEL = 269
    JOIN_EXPEDITION = 270
    DECLINE_EXPEDITION_INVITATION = 271
    VOTE_START = 272
    VOTE_DO_VOTE = 273
    RATING_DO_RATE = 274
    ENTERING_EXPEDITION_START = 275
    ENTERING_EXPEDITION_CANCEL = 276
    ACTIVATE_EXPEDITION_CHECK_POINT = 277
    ARENA_REGISTER = 278
    ARENA_ADD_INVITE = 279
    ARENA_REGISTER_CANCEL = 280
    ARENA_LEAVE = 281
    JOIN_ARENA_MATCH = 282
    DECLINE_ARENA_INVITATION = 283
    ENTERING_ARENA_START = 284
    ENTERING_ARENA_CANCEL = 285
    ARENA_CUSTOM_MATCH = 286
    UPDATE_CHARACTER_STATEMENT = 287
    BOOST_FARMABLE = 288
    GET_STRIKE_HISTORY = 289
    USE_FUNCTION = 290
    USE_PORTAL_ENTRANCE = 291
    RESET_PORTAL_BINDING = 292
    QUERY_PORTAL_BINDING = 293
    CLAIM_PAYMENT_TRANSACTION = 294
    CHANGE_USE_FLAG = 295
    CLIENT_PERFORMANCE_STATS = 296
    EXTENDED_HARDWARE_STATS = 297
    CLIENT_LOW_MEMORY_WARNING = 298
    TERRITORY_CLAIM_START = 299
    TERRITORY_CLAIM_CANCEL = 300
    DELIVER_CARRIABLE_OBJECT_START = 301
    DELIVER_CARRIABLE_OBJECT_CANCEL = 302
    TERRITORY_UPGRADE_WITH_POWER_CRYSTAL = 303
    REQUEST_APP_STORE_PRODUCTS = 304
    VERIFY_PRODUCT_PURCHASE = 305
    QUERY_GUILD_PLAYER_STATS = 306
    QUERY_ALLIANCE_GUILD_STATS = 307
    TRACK_ACHIEVEMENTS = 308
    SET_ACHIEVEMENTS_AUTO_LEARN = 309
    DEPOSIT_ITEM_TO_GUILD_CURRENCY = 310
    WITHDRAWAL_ITEM_FROM_GUILD_CURRENCY = 311
    AUCTION_SELL_SPECIFIC_ITEM_REQUEST = 312
    FISHING_START = 313
    FISHING_CASTING = 314
    FISHING_CAST = 315
    FISHING_CATCH = 316
    FISHING_PULL = 317
    FISHING_GIVE_LINE = 318
    FISHING_FINISH = 319
    FISHING_CANCEL = 320
    CREATE_GUILD_ACCESS_TAG = 321
    DELETE_GUILD_ACCESS_TAG = 322
    RENAME_GUILD_ACCESS_TAG = 323
    FLAG_GUILD_ACCESS_TAG_GUILD_PERMISSION = 324
    ASSIGN_GUILD_ACCESS_TAG = 325
    REMOVE_GUILD_ACCESS_TAG_FROM_PLAYER = 326
    MODIFY_GUILD_ACCESS_TAG_EDITORS = 327
    REQUEST_PUBLIC_ACCESS_TAGS = 328
    CHANGE_ACCESS_TAG_PUBLIC_FLAG = 329
    UPDATE_GUILD_ACCESS_TAG = 330
    STEAM_START_MICROTRANSACTION = 331
    STEAM_FINISH_MICROTRANSACTION = 332
    STEAM_ID_HAS_ACTIVE_ACCOUNT = 333
    CHECK_EMAIL_ACCOUNT_STATE = 334
    LINK_ACCOUNT_TO_STEAM_ID = 335
    IN_APP_CONFIRM_PAYMENT_GOOGLE_PLAY = 336
    IN_APP_CONFIRM_PAYMENT_APPLE_APP_STORE = 337
    IN_APP_PURCHASE_REQUEST = 338
    IN_APP_PURCHASE_FAILED = 339
    CHARACTER_SUBSCRIPTION_INFO = 340
    ACCOUNT_SUBSCRIPTION_INFO = 341
    BUY_GVG_SEASON_BOOSTER = 342
    CHANGE_FLAGGING_PREPARE = 343
    OVER_CHARGE = 344
    OVER_CHARGE_END = 345
    REQUEST_TRUSTED = 346
    CHANGE_GUILD_LOGO = 347
    PARTY_FINDER_REGISTER_FOR_UPDATES = 348
    PARTY_FINDER_UNREGISTER_FOR_UPDATES = 349
    PARTY_FINDER_ENLIST_NEW_PARTY_SEARCH = 350
    PARTY_FINDER_DELETE_PARTY_SEARCH = 351
    PARTY_FINDER_CHANGE_PARTY_SEARCH = 352
    PARTY_FINDER_CHANGE_ROLE = 353
    PARTY_FINDER_APPLY_FOR_GROUP = 354
    PARTY_FINDER_ACCEPT_OR_DECLINE_APPLY_FOR_GROUP = 355
    PARTY_FINDER_GET_EQUIPMENT_SNAPSHOT = 356
    PARTY_FINDER_REGISTER_APPLICANTS = 357
    PARTY_FINDER_UNREGISTER_APPLICANTS = 358
    PARTY_FINDER_FULLTEXT_SEARCH = 359
    PARTY_FINDER_REQUEST_EQUIPMENT_SNAPSHOT = 360
    GET_PERSONAL_SEASON_TRACKER_DATA = 361
    GET_PERSONAL_SEASON_PAST_REWARD_DATA = 362
    USE_CONSUMABLE_FROM_INVENTORY = 363
    CLAIM_PERSONAL_SEASON_REWARD = 364
    EASY_ANTI_CHEAT_MESSAGE_TO_SERVER = 365
    XIGN_CODE_MESSAGE_TO_SERVER = 366
    BATTL_EYE_MESSAGE_TO_SERVER = 367
    SET_NEXT_TUTORIAL_STATE = 368
    ADD_PLAYER_TO_MUTE_LIST = 369
    REMOVE_PLAYER_FROM_MUTE_LIST = 370
    PRODUCT_SHOP_USER_EVENT = 371
    GET_VANITY_UNLOCKS = 372
    BUY_VANITY_UNLOCKS = 373
    GET_MOUNT_SKINS = 374
    SET_MOUNT_SKIN = 375
    SET_WARDROBE = 376
    CHANGE_CUSTOMIZATION = 377
    CHANGE_PLAYER_ISLAND_DATA = 378
    GET_GUILD_CHALLENGE_POINTS = 379
    SMART_QUEUE_JOIN = 380
    SMART_QUEUE_LEAVE = 381
    SMART_QUEUE_SELECT_SPAWN_CLUSTER = 382
    UPGRADE_HIDEOUT = 383
    INIT_HIDEOUT_ATTACK_START = 384
    INIT_HIDEOUT_ATTACK_CANCEL = 385
    HIDEOUT_FILL_NUTRITION = 386
    HIDEOUT_GET_INFO = 387
    HIDEOUT_GET_OWNER_INFO = 388
    HIDEOUT_SET_TRIBUTE = 389
    HIDEOUT_UPGRADE_WITH_POWER_CRYSTAL = 390
    HIDEOUT_DECLARE_H_Q = 391
    HIDEOUT_UNDECLARE_H_Q = 392
    HIDEOUT_GET_H_Q_REQUIREMENTS = 393
    HIDEOUT_BOOST = 394
    HIDEOUT_BOOST_CONSTRUCTION = 395
    OPEN_WORLD_ATTACK_SCHEDULE_START = 396
    OPEN_WORLD_ATTACK_SCHEDULE_CANCEL = 397
    OPEN_WORLD_ATTACK_CONQUER_START = 398
    OPEN_WORLD_ATTACK_CONQUER_CANCEL = 399
    GET_OPEN_WORLD_ATTACK_DETAILS = 400
    GET_NEXT_OPEN_WORLD_ATTACK_SCHEDULE_TIME = 401
    RECOVER_VAULT_FROM_HIDEOUT = 402
    GET_GUILD_ENERGY_DRAIN_INFO = 403
    CHANNELING_UPDATE = 404
    USE_CORRUPTED_SHRINE = 405
    REQUEST_ESTIMATED_MARKET_VALUE = 406
    LOG_FEEDBACK = 407
    GET_INFAMY_INFO = 408
    GET_PARTY_SMART_CLUSTER_QUEUE_PRIORITY = 409
    SET_PARTY_SMART_CLUSTER_QUEUE_PRIORITY = 410
    CLIENT_ANTI_AUTO_CLICKER_INFO = 411
    CLIENT_BOT_PATTERN_DETECTION_INFO = 412
    CLIENT_ANTI_GATHER_CLICKER_INFO = 413
    LOADOUT_CREATE = 414
    LOADOUT_READ = 415
    LOADOUT_READ_HEADERS = 416
    LOADOUT_UPDATE = 417
    LOADOUT_DELETE = 418
    LOADOUT_ORDER_UPDATE = 419
    LOADOUT_EQUIP = 420
    BATCH_USE_ITEM_CANCEL = 421
    ENLIST_FACTION_WARFARE = 422
    GET_FACTION_WARFARE_WEEKLY_REPORT = 423
    CLAIM_FACTION_WARFARE_WEEKLY_REPORT = 424
    GET_FACTION_WARFARE_CAMPAIGN_DATA = 425
    CLAIM_FACTION_WARFARE_ITEM_REWARD = 426
    SEND_MEMORY_CONSUMPTION = 427
    PICKUP_CARRIABLE_OBJECT_START = 428
    PICKUP_CARRIABLE_OBJECT_CANCEL = 429
    SET_SAVING_CHEST_LOGS_FLAG = 430
    GET_SAVING_CHEST_LOGS_FLAG = 431
    REGISTER_GUEST_ACCOUNT = 432
    RESEND_GUEST_ACCOUNT_VERIFICATION_EMAIL = 433
    DO_SIMPLE_ACTION_START = 434
    DO_SIMPLE_ACTION_CANCEL = 435
    GET_GVG_SEASON_CONTRIBUTION_BY_ACTIVITY = 436
    GET_GVG_SEASON_CONTRIBUTION_BY_CRYSTAL_LEAGUE = 437
    GET_GUILD_MIGHT_CATEGORY_CONTRIBUTION = 438
    GET_GUILD_MIGHT_CATEGORY_OVERVIEW = 439
    GET_PVP_CHALLENGE_DATA = 440
    CLAIM_PVP_CHALLENGE_WEEKLY_REWARD = 441
    GET_PERSONAL_MIGHT_STATS = 442
    AUCTION_GET_LOADOUT_OFFERS = 443
    AUCTION_BUY_LOADOUT_OFFER = 444
    ACCOUNT_DELETION_REQUEST = 445
    ACCOUNT_REACTIVATION_REQUEST = 446
    GET_MODERATION_ESCALATION_DEFINITON = 447
    EVENT_BASED_POPUP_ADD_SEEN = 448
    GET_ITEM_KILL_HISTORY = 449
    GET_VANITY_CONSUMABLES = 450
    EQUIP_KILL_EMOTE = 451
    CHANGE_KILL_EMOTE_PLAY_ON_KNOCKDOWN_SETTING = 452
    BUY_VANITY_CONSUMABLE_CHARGES = 453
    RECLAIM_VANITY_ITEM = 454
    GET_ARENA_RANKINGS = 455
    GET_CRYSTAL_LEAGUE_STATISTICS = 456
    SEND_OPTIONS_LOG = 457
    SEND_CONTROLS_OPTIONS_LOG = 458
    MISTS_USE_IMMEDIATE_RETURN_EXIT = 459
    MISTS_USE_STATIC_ENTRANCE = 460
    MISTS_USE_CITY_ROADS_ENTRANCE = 461
    CHANGE_NEW_GUILD_MEMBER_MAIL = 462
    GET_NEW_GUILD_MEMBER_MAIL = 463
    CHANGE_GUILD_FACTION_ALLEGIANCE = 464
    GET_GUILD_FACTION_ALLEGIANCE = 465
    GUILD_BANNER_CHANGE = 466
    GUILD_GET_OPTIONAL_STATS = 467
    GUILD_SET_OPTIONAL_STATS = 468
    GET_PLAYER_INFO_FOR_STALK = 469
    PAY_GOLD_FOR_CHARACTER_TYPE_CHANGE = 470
    QUICK_SELL_AUCTION_QUERY_ACTION = 471
    QUICK_SELL_AUCTION_SELL_ACTION = 472
    FCM_TOKEN_TO_SERVER = 473
    APNS_TOKEN_TO_SERVER = 474
    DEATH_RECAP = 475
    AUCTION_FETCH_FINISHED_AUCTIONS = 476
    ABORT_AUCTION_FETCH_FINISHED_AUCTIONS = 477
    REQUEST_LEGENDARY_EVEN_HISTORY = 478
    PARTY_ANSWER_START_HUNT_REQUEST = 479
    HUNT_ABORT = 480
    USE_FIND_TRACK_SPELL_FROM_ITEM_PREPARE = 481
    INTERACT_WITH_TRACK_START = 482
    INTERACT_WITH_TRACK_CANCEL = 483
    TERRITORY_RAID_START = 484
    TERRITORY_RAID_CANCEL = 485
    TERRITORY_CLAIM_RAIDED_RAW_ENERGY_CRYSTAL_RESULT = 486
    GV_G_SEASON_PLAYER_GUILD_PARTICIPATION_DETAILS = 487
    DAILY_MIGHT_BONUS = 488
    CLAIM_DAILY_MIGHT_BONUS = 489
    GET_FORTIFICATION_GROUP_INFO = 490
    UPGRADE_FORTIFICATION_GROUP = 491
    CANCEL_UPGRADE_FORTIFICATION_GROUP = 492
    DOWNGRADE_FORTIFICATION_GROUP = 493
    GET_CLUSTER_ACTIVITY_CHEST_ESTIMATES = 494
    PARTY_READY_CHECK_BEGIN = 495
    PARTY_READY_CHECK_UPDATE = 496
    CLAIM_ALBION_JOURNAL_REWARD = 497
    TRACK_ALBION_JOURNAL_ACHIEVEMENTS = 498
    REQUEST_OUTLANDS_TELEPORTATION_USAGE = 499



from albibong.classes.character import Character
from albibong.classes.event_handler.world_data_utils import WorldDataUtils
from albibong.classes.world_data import WorldData
from albibong.threads.websocket_server import send_event


def handle_event_health_update(world_data: WorldData, parameters):

    if world_data.is_dps_meter_running == False:
        return

    if 0 not in parameters or 6 not in parameters or 2 not in parameters:
        return

    nominal = parameters[2]
    target = parameters[0]
    inflictor = parameters[6]

    WorldDataUtils.update_damage_or_heal(world_data, target, inflictor, nominal)


def handle_event_health_updates(world_data: WorldData, parameters):

    if world_data.is_dps_meter_running == False:
        return

    if 0 not in parameters or 6 not in parameters or 2 not in parameters:
        return

    for i in range(len(parameters[2])):
        nominal = parameters[2][i]
        target = parameters[0]
        inflictor = parameters[6][i]

        WorldDataUtils.update_damage_or_heal(world_data, target, inflictor, nominal)
